#!/usr/bin/env python
#
# Package Name: ewccli
# License: GPL-3.0-or-later
# Copyright (c) 2025 EUMETSAT, ECMWF for European Weather Cloud
# See the LICENSE file for more details

import os
from pathlib import Path

from ewccli.enums import Federee, FedereeDNSMapping


class EWCCLIConfiguration:
    """EWC CLI global configuration."""

    EWC_CLI_NAME = "ewc"
    EWC_CLI_DEBUG_LEVEL = os.getenv("EWC_CLI_DEBUG_LEVEL", "INFO")
    EWC_CLI_DRY_RUN = bool(int(os.getenv("EWC_CLI_DRY_RUN", 0)))

    EWC_CLI_HUB_ITEMS_URL = "https://raw.githubusercontent.com/ewcloud/ewc-community-hub/refs/heads/main/items.yaml"
    EWC_CLI_HUB_DOWNLOAD_ITEMS = bool(int(os.getenv("EWC_CLI_HUB_DOWNLOAD_ITEMS", 0)))

    home_dir = Path.home()

    EWC_CLI_BASE_PATH = home_dir / ".ewccli"

    EWC_CLI_PROFILES_PATH = EWC_CLI_BASE_PATH / "profiles"

    EWC_CLI_DEFAULT_PROFILE_NAME = "default"
    EWC_CLI_DEFAULT_FEDEREE = "default"
    EWC_CLI_DEFAULT_KEYPAIR_NAME = "ewc-hub-key"

    # EWC_CLI_HUB_ITEMS_PATH = files("ewccli.data").joinpath("items.yaml")
    EWC_CLI_HUB_ITEMS_PATH = EWC_CLI_BASE_PATH / "items.yaml"
    EWC_CLI_HUB_SSH_REPO_PATH = home_dir / ".ssh"
    EWC_CLI_PRIVATE_SSH_KEY_PATH = EWC_CLI_HUB_SSH_REPO_PATH / "id_rsa"
    EWC_CLI_PUBLIC_SSH_KEY_PATH = EWC_CLI_HUB_SSH_REPO_PATH / "id_rsa.pub"

    EWC_CLI_DEFAULT_PATH_INPUTS = EWC_CLI_BASE_PATH / "inputs"
    EWC_CLI_DEFAULT_PATH_OUTPUTS = EWC_CLI_BASE_PATH / "outputs"

    # CPU images
    EWC_CLI_CPU_IMAGES = [
        "Rocky-8",
        "Rocky-9",
        "Ubuntu-22.04",
        "Ubuntu-24.04",
    ]

    EWC_CLI_DEFAULT_IMAGE = "Rocky-9"

    EWC_CLI_IMAGES_USER = {
        "Ubuntu-22.04": "ubuntu",
        "Ubuntu-24.04": "ubuntu",
        "Rocky-8": "cloud-user",
        "Rocky-9": "cloud-user",
        "Rocky-9-GPU": "cloud-user",
        "Ubuntu 22.04 NVIDIA_AI": "eouser",

    }

    EWC_CLI_AUTH_URL_MAP = {
        "https://auth.os-api.cci1.ecmwf.int:443": Federee.ECMWF.value,
        "https://keystone.cloudferro.com:5000": Federee.EUMETSAT.value,
    }

    EWC_CLI_SITE_MAP = {
        Federee.ECMWF.value: "https://auth.os-api.cci1.ecmwf.int:443",
        Federee.EUMETSAT.value: "https://keystone.cloudferro.com:5000",
    }

    # GPU images short custom names
    EWC_CLI_GPU_IMAGES_SITE_MAP = {
        Federee.ECMWF.value: "Rocky-9-GPU",
        Federee.EUMETSAT.value: "Ubuntu-22.04-GPU",
    }

    # GPU images
    EWC_CLI_GPU_IMAGES = [v for _,v in EWC_CLI_GPU_IMAGES_SITE_MAP.items()]


    # Openstack value of the GPU images
    EWC_CLI_OS_GPU_IMAGES_SITE_MAP = {
        Federee.ECMWF.value: "Rocky-9.6-GPU",  # This can be find after normalization
        Federee.EUMETSAT.value: "Ubuntu 22.04 NVIDIA_AI", # ( usually fixed)
    }

    # Flavors

    # CPU
    # TODO: Create config list
    DEFAULT_CPU_FLAVOURS_MAP = {
        Federee.ECMWF.value: "4cpu-4gbmem-30gbdisk",
        Federee.EUMETSAT.value: "eo1.large",
    }

    # GPU
    # TODO: To be removed once the harmonization on flavours is finalized
    GPU_FLAVOURS_MAP = {
        Federee.ECMWF.value: [
            "2cpu-4gbmem-30gbdisk",
            "8cpu-64gbmem-30gbdisk-a100.2g.20gbgpu",
            "16cpu-128gbmem-30gbdisk-40gbgpu",
            "48cpu-384gbmem-30gbdisk-80gbgpu",
        ],
        Federee.EUMETSAT.value: [
            "vm.a6000.1",
            "vm.a6000.2",
            "vm.a6000.4",
            "vm.a6000.8",
        ],
    }

    DEFAULT_GPU_FLAVOURS_MAP = {
        Federee.ECMWF.value: "8cpu-64gbmem-30gbdisk-a100.1g.10gbgpu",
        Federee.EUMETSAT.value: "vm.a6000.2",
    }

    # Network
    
    DEFAULT_NETWORK_MAP = {
        Federee.ECMWF.value: "private",
        Federee.EUMETSAT.value: "private",
    }

    DEFAULT_EXTERNAL_NETWORK_MAP = {
        Federee.ECMWF.value: "external-internet",
        Federee.EUMETSAT.value: "external",
    }

    DNS_CHECK_TIMEOUT_MINUTES = 20
    FEDEREE_DNS_MAPPING = {
        Federee.ECMWF.value: FedereeDNSMapping.ECMWF.value,
        Federee.EUMETSAT.value: FedereeDNSMapping.EUMETSAT.value,
    }

    DEFAULT_SECURITY_GROUP_MAP = {
        Federee.ECMWF.value: ("ssh",),
        Federee.EUMETSAT.value: ("ssh",),
    }

    # Crossplane configurations
    DEFAULT_KUBERNETES_SERVER = {
        Federee.ECMWF.value: "",
        Federee.EUMETSAT.value: "",
    }


config = EWCCLIConfiguration()



#!/usr/bin/env python
#
# Package Name: ewccli
# License: GPL-3.0-or-later
# Copyright (c) 2025 EUMETSAT, ECMWF for European Weather Cloud
# See the LICENSE file for more details

"""Initialize ewccli package."""

__title__ = "ewccli"
__version__ = "0.3.0"
__author__ = "European Weather Cloud <support@europeanweather.cloud>"

__all__ = [
    "__title__",
    "__version__",
]



#!/usr/bin/env python
#
# Package Name: ewccli
# License: GPL-3.0-or-later
# Copyright (c) 2025 EUMETSAT, ECMWF for European Weather Cloud
# See the LICENSE file for more details

"""Common CRD classes."""


class GroupVersionResource:
    """
    Represents a Kubernetes GroupVersionResource (GVR) identifier.

    Attributes:
        group (str): The API group of the resource (e.g., "apps", "dns.europeanweather.cloud").
        version (str): The API version of the resource (e.g., "v1", "v1alpha1").
        resource (str): The plural name of the resource (e.g., "pods", "records").
    """

    def __init__(self, group: str, version: str, resource: str):
        self.group = group
        self.version = version
        self.resource = resource



#!/usr/bin/env python
#
# Package Name: ewccli
# License: GPL-3.0-or-later
# Copyright (c) 2025 EUMETSAT, ECMWF for European Weather Cloud
# See the LICENSE file for more details


"""Tests for EWC login command."""


import pytest
from pathlib import Path
from click import ClickException

from ewccli.commands.login_command import check_and_generate_ssh_keys


# -----------------------------
# Case 1: both keys exist & match
# -----------------------------
def test_existing_matching_keys(tmp_path, monkeypatch):
    priv = tmp_path / "id_rsa"
    pub = tmp_path / "id_rsa.pub"

    priv.write_text("private")
    pub.write_text("public")

    # Patch where function is USED
    monkeypatch.setattr(
        "ewccli.commands.login_command.ssh_keys_match",
        lambda ssh_private_key_path, ssh_public_key_path: True,
    )

    result_priv, result_pub = check_and_generate_ssh_keys(
        ssh_public_key_path=str(pub),
        ssh_private_key_path=str(priv),
        resolved_profile="testprofile",
    )

    assert result_priv == str(priv)
    assert result_pub == str(pub)


# -----------------------------
# Case 1b: keys exist but mismatch
# -----------------------------
def test_existing_mismatching_keys(tmp_path, monkeypatch):
    priv = tmp_path / "id_rsa"
    pub = tmp_path / "id_rsa.pub"

    priv.write_text("private")
    pub.write_text("public")

    monkeypatch.setattr(
        "ewccli.commands.login_command.ssh_keys_match",
        lambda ssh_private_key_path, ssh_public_key_path: False,
    )

    with pytest.raises(ClickException):
        check_and_generate_ssh_keys(
            ssh_public_key_path=str(pub),
            ssh_private_key_path=str(priv),
            resolved_profile="testprofile",
        )


# -----------------------------
# Case 2: both keys missing -> user generates
# -----------------------------
def test_missing_keys_generate(tmp_path, monkeypatch):
    priv = tmp_path / "id_rsa"
    pub = tmp_path / "id_rsa.pub"

    # Patch click.confirm in login_command
    monkeypatch.setattr(
        "ewccli.commands.login_command.click.confirm",
        lambda *args, **kwargs: True,
    )

    # Fake generate now only takes resolved_profile
    def fake_generate(resolved_profile):
        priv.write_text("generated private")
        pub.write_text("generated public")
        return priv, pub

    monkeypatch.setattr(
        "ewccli.commands.login_command.generate_ssh_keypair",
        fake_generate,
    )

    result_priv, result_pub = check_and_generate_ssh_keys(
        ssh_public_key_path=str(pub),
        ssh_private_key_path=str(priv),
        resolved_profile="profile",
    )

    assert Path(result_priv).exists()
    assert Path(result_pub).exists()


# -----------------------------
# Case 2b: both missing but user refuses generation
# -----------------------------
def test_missing_keys_user_declines(tmp_path, monkeypatch):
    priv = tmp_path / "id_rsa"
    pub = tmp_path / "id_rsa.pub"

    monkeypatch.setattr(
        "ewccli.commands.login_command.click.confirm",
        lambda *args, **kwargs: False,
    )

    with pytest.raises(ClickException):
        check_and_generate_ssh_keys(
            ssh_public_key_path=str(pub),
            ssh_private_key_path=str(priv),
            resolved_profile="profile",
        )


# -----------------------------
# Case 3: only one key exists
# -----------------------------
def test_only_private_key_exists(tmp_path):
    priv = tmp_path / "id_rsa"
    pub = tmp_path / "id_rsa.pub"

    priv.write_text("private")

    with pytest.raises(ClickException):
        check_and_generate_ssh_keys(
            ssh_public_key_path=str(pub),
            ssh_private_key_path=str(priv),
            resolved_profile="profile",
        )


def test_only_public_key_exists(tmp_path):
    priv = tmp_path / "id_rsa"
    pub = tmp_path / "id_rsa.pub"

    pub.write_text("public")

    with pytest.raises(ClickException):
        check_and_generate_ssh_keys(
            ssh_public_key_path=str(pub),
            ssh_private_key_path=str(priv),
            resolved_profile="profile",
        )



#!/usr/bin/env python
#
# Package Name: ewccli
# License: GPL-3.0-or-later
# Copyright (c) 2025 EUMETSAT, ECMWF for European Weather Cloud
# See the LICENSE file for more details


"""CLI EWC Login: EWC Login interaction."""

import os
import re
from pathlib import Path

import rich_click as click
from rich.console import Console
from click import ClickException

from prompt_toolkit.application import Application
from prompt_toolkit.widgets import RadioList, Box, Frame
from prompt_toolkit.layout import Layout
from prompt_toolkit.styles import Style

from kubernetes import config
from kubernetes.config.config_exception import (  # noqa: N813
    ConfigException as kubernetes_config_exception,
)
from openstack.config import OpenStackConfig
from openstack.exceptions import (  # noqa: N813
    ConfigException as openstack_config_exception,
)

from ewccli.configuration import config as ewc_hub_config
from ewccli.utils import save_cli_profile, _resolve_profile
from ewccli.utils import generate_ssh_keypair, ssh_keys_match
from ewccli.utils import save_default_login_profile
from ewccli.enums import Federee
from ewccli.logger import get_logger

_LOGGER = get_logger(__name__)


console = Console()


def kubeconfig_available():
    """Verify if kubeconfig is available."""
    try:
        config.load_kube_config()
        return True
    except kubernetes_config_exception as e:
        _LOGGER.warning(
            f"⚠️ Kubeconfig not found: {e}\n"
            "You could set KUBECONFIG=/path/to/your/kubeconfig or continue below using the token"
        )
        return False


def cloud_yaml_exists():
    """Check if OpenStack clouds.yaml file exists."""
    # Default OpenStack config paths (can vary by environment)
    default_paths = [
        Path(
            os.getenv("OS_CLIENT_CONFIG_FILE", "~/.config/openstack/clouds.yaml")
        ).expanduser(),
        Path("/etc/openstack/clouds.yaml"),
    ]

    return any(p.exists() for p in default_paths)


def openstack_config_available():
    """Verify if OpenStack cloud config is available."""
    try:
        os_config = OpenStackConfig()
        if cloud_yaml_exists():
            os_config.get_one_cloud()
        else:
            _LOGGER.warning(
                "⚠️ OpenStack cloud config not found at '~/.config/openstack/cloud.yaml'\n"
                "You can set the config path with the environment variable:\n"
                "  OS_CLIENT_CONFIG_FILE=/path/to/clouds.yaml\n"
                "Alternatively, provide your credentials using:\n"
                "  OS_APPLICATION_CREDENTIAL_ID and OS_APPLICATION_CREDENTIAL_SECRET\n"
                "Or continue below to enter them manually."
            )
            return False
        return True
    except openstack_config_exception as e:
        _LOGGER.warning(
            f"⚠️ OpenStack cloud config not found: {e}\n"
            "You can also set the config path with `OS_CLIENT_CONFIG_FILE=/path/to/clouds.yaml` or continue below"
        )
        return False


def validate_tenant_name(ctx, param, value):
    """Validate tenant name."""
    pattern = r"^[a-zA-Z0-9]+-[a-zA-Z0-9]+-[a-zA-Z0-9]+$"
    if not re.match(pattern, value):
        raise click.BadParameter(
            "Config name must be exactly 3 alphanumeric parts separated by dashes (e.g. thisis-my-tenancy)."
        )
    return value


def init_options(func):
    """Login options for the CLI login command."""
    func = click.option(
        "--tenant-name",
        envvar="EWC_CLI_LOGIN_TENANT_NAME",
        prompt=True,
        required=True,
        callback=validate_tenant_name,
        help=(
            "Name of your tenancy in EWC, used to identify cloud configurations.\n"
            "Must follow the format: 'part1-part2-part3' (e.g. 'demo-user-eu'), "
            "where each part is alphanumeric and separated by dashes.\n"
            "Can also be set via the EWC_CLI_LOGIN_TENANT_NAME environment variable."
        ),
    )(func)
    func = click.option(
        "--federee",
        type=click.Choice(
            [Federee.ECMWF.value, Federee.EUMETSAT.value], case_sensitive=True
        ),
        envvar="EWC_CLI_LOGIN_REGION",
        help=(
            "Cloud federee where the resources will be deployed. "
            "You can also set this using the EWC_CLI_LOGIN_REGION environment variable. "
            "If not provided, you'll be prompted to choose."
        ),
    )(func)
    func = click.option(
        "--application-credential-id",
        required=False,
        hide_input=True,
        help=(
            "OpenStack Application Credential ID. "
            "Ignored if environment variable OS_APPLICATION_CREDENTIAL_ID is set, "
            "or if a cloud.yaml config is found at '~/.config/openstack/cloud.yaml' "
            "or at the path specified by OS_CLIENT_CONFIG_FILE."
        ),
    )(func)
    func = click.option(
        "--application-credential-secret",
        required=False,
        hide_input=True,
        help=(
            "OpenStack Application Credential Secret. "
            "Ignored if environment variable OS_APPLICATION_CREDENTIAL_SECRET is set, "
            "or if a cloud.yaml config is found at '~/.config/openstack/cloud.yaml' "
            "or at the path specified by OS_CLIENT_CONFIG_FILE."
        ),
    )(func)
    # func = click.option(
    #     "--token",
    #     hide_input=True,
    #     required=False,
    #     default="",
    #     help=(
    #         "Kubernetes token (leave blank if not needed).\n"
    #         "Provide this only if you plan to use Kubernetes services and "
    #         "do not have a kubeconfig file available "
    #         "(e.g. ~/.kube/config or via the KUBECONFIG environment variable)."
    #     ),
    # )(func)
    func = click.option(
        "--ssh-public-key-path",
        required=False,
        envvar="EWC_CLI_SSH_PUBLIC_KEY_PATH",
        type=str,
        default=ewc_hub_config.EWC_CLI_PUBLIC_SSH_KEY_PATH,
        show_default=True,
        help="Path to SSH public key.",
    )(func)
    func = click.option(
        "--ssh-private-key-path",
        required=False,
        envvar="EWC_CLI_SSH_PRIVATE_KEY_PATH",
        type=str,
        default=ewc_hub_config.EWC_CLI_PRIVATE_SSH_KEY_PATH,
        show_default=True,
        help="Path to SSH private key.",
    )(func)
    func = click.option(
        "--profile",
        envvar="EWC_CLI_LOGIN_PROFILE",
        required=False,
        help="EWC CLI profile name",
    )(func)
    return func


def select_provider():
    """Select provider."""
    choices = [
        ("EUMETSAT", "EUMETSAT"),
        ("ECMWF", "ECMWF"),
    ]

    radio_list = RadioList(choices)

    # Use the widget's own default key bindings
    kb = radio_list.control.key_bindings

    @kb.add("enter")
    def _(event):
        index = radio_list._selected_index
        selected_value = radio_list.values[index][
            1
        ]  # values is list of tuples (display, value)
        event.app.exit(result=selected_value)

    # Add quit keys as well
    @kb.add("c-c")
    @kb.add("c-q")
    def _(event):
        event.app.exit(None)

    root_container = Box(Frame(radio_list, title="Select Federee"), padding=1)
    layout = Layout(root_container)

    app = Application(
        layout=layout,
        key_bindings=kb,
        full_screen=False,
        mouse_support=True,
        style=Style.from_dict(
            {
                "frame.label": "bold",
            }
        ),
    )

    selected = app.run()
    return selected


def check_and_generate_ssh_keys(
    ssh_public_key_path: str,
    ssh_private_key_path: str,
    resolved_profile: str
):
    """Check for SSH keys, prompt to generate if missing"""
    private_exists = Path(ssh_private_key_path).exists()
    public_exists = Path(ssh_public_key_path).exists()

    if private_exists and public_exists:
        # case 1: both exist
        console.print(
            "Using the following path for the SSH keypair:"
            f"\nSSH public key path: {ssh_public_key_path}"
            f"\nSSH private key path: {ssh_private_key_path}\n"
        )
        console.print("SSH key pair exists, checking consitency...")

        is_matching = ssh_keys_match(
            ssh_private_key_path=ssh_private_key_path,
            ssh_public_key_path=ssh_public_key_path
        )

        if not is_matching:
            raise ClickException(
                "SSH keys provided are not a correct keypair:"
                f"\nSSH public key path: {ssh_public_key_path}"
                f"\nSSH private key path: {ssh_private_key_path}"
                "\nMake sure either you pass correct SSH keypair in the EWC login command through the following flags `--ssh-private-key-path` and `--ssh-public-key-path`"
                "or let the `ewc login` command create them for you. Exiting."
            )
        else:
            click.secho("SSH private and public keys are matching! Continuing...", fg="green")
        
        return ssh_private_key_path, ssh_public_key_path

    elif not private_exists and not public_exists:
        # case 2: neither exists
        console.print(
            "SSH key pair is missing:"
            f"\nSSH public key path: {ssh_public_key_path}"
            f"\nSSH private key path: {ssh_private_key_path}\n"
        )

        if click.confirm("Do you want to generate a new SSH key pair?", default=False):
            ssh_custom_private_key_path, ssh_custom_public_key_path = generate_ssh_keypair(
                resolved_profile=resolved_profile
            )
            return ssh_custom_private_key_path, ssh_custom_public_key_path
        else:
            raise ClickException(
                "SSH key generation skipped but SSH keys are mandatory to deploy VMs or hub items."
                " Make sure either you pass SSH keys in the EWC login command through the following flags `--ssh-private-key-path` and `--ssh-public-key-path`"
                "or let the `ewc login` command create them for you. Exiting."
            )

    else:
        # case 3: exactly one exists
        if private_exists and not public_exists:
            key_exists = "public"
            key_path = ssh_public_key_path

        if not private_exists and public_exists:
            key_exists = "private"
            key_path = ssh_private_key_path

        raise ClickException(
            f"SSH {key_exists} key is missing at: {key_path}."
            " Make sure the keypair is passed!"
            " You can pass SSH keys in the EWC login command through the following flags `--ssh-private-key-path` or `--ssh-public-key-path`"
            "or let the `ewc login` command create them for you. Exiting."
        )


def init_command(
    application_credential_id: str,
    application_credential_secret: str,
    ssh_public_key_path: str,
    ssh_private_key_path: str,
    tenant_name: str,
    federee: str,
    profile: str = None,
    # token: str,
):
    """EWC CLI Login."""
    if not federee:
        # If --federee is not passed, ask interactively
        federee = select_provider()
        if not federee:
            console.print("No selection made. Exiting.")
            return

    console.print(f"Considering federee: {federee}")

    resolved_profile = _resolve_profile(profile, federee, tenant_name)

    ssh_private_key_path_to_save, ssh_public_key_path_to_save = check_and_generate_ssh_keys(
        ssh_public_key_path=ssh_public_key_path,
        ssh_private_key_path=ssh_private_key_path,
        resolved_profile=resolved_profile,
    )

    if openstack_config_available():
        console.print(
            "🔑 [bold green]Openstack cloud.yaml found at ~/.config/openstack/clouds.yaml[/bold green]"
            " – skipping Openstack ID and secret requirements."
        )
        application_credential_id = ""
        application_credential_secret = ""

    elif not application_credential_id or not application_credential_secret:
        if not application_credential_id:
            # Handle OpenStack credential ID
            application_credential_id = (
                application_credential_id
                or os.getenv("OS_APPLICATION_CREDENTIAL_ID")
                or click.prompt(
                    "Enter OpenStack Application Credential ID", hide_input=True
                )
            )

        if not application_credential_secret:
            # Handle OpenStack credential secret
            application_credential_secret = (
                application_credential_secret
                or os.getenv("OS_APPLICATION_CREDENTIAL_SECRET")
                or click.prompt(
                    "Enter OpenStack Application Credential Secret", hide_input=True
                )
            )

    # if kubeconfig_available():
    #     click.echo("🔑 kubeconfig found – skipping token requirement.")
    #     token = None
    # elif not token:
    #     token = click.prompt(
    #         "Enter Kubernetes token (leave blank if not needed)",
    #         hide_input=True,
    #         default="",
    #         show_default=False,
    #         prompt_suffix=": ",
    #     )
    #     if token == "":
    #         token = None

    #
    save_default_login_profile(
        federee=federee,
        tenant_name=tenant_name,
        ssh_private_key_path_to_save=ssh_private_key_path_to_save,
        ssh_public_key_path_to_save=ssh_public_key_path_to_save,
        # token=token,
        application_credential_id=application_credential_id,
        application_credential_secret=application_credential_secret,
    )

    # Save config
    save_cli_profile(
        federee=federee,
        tenant_name=tenant_name,
        ssh_private_key_path_to_save=ssh_private_key_path_to_save,
        ssh_public_key_path_to_save=ssh_public_key_path_to_save,
        profile=profile,
        # token=token,
        application_credential_id=application_credential_id,
        application_credential_secret=application_credential_secret,
    )

    console.print(
        f"✅ Profile '[bold cyan]{resolved_profile}[/bold cyan]' saved "
        f"in the following file {ewc_hub_config.EWC_CLI_PROFILES_PATH}"
    )



#!/usr/bin/env python
#
# Package Name: ewccli
# License: GPL-3.0-or-later
# Copyright (c) 2025 EUMETSAT, ECMWF for European Weather Cloud
# See the LICENSE file for more details


"""Test download item methods."""

import pytest
import requests

from ewccli.configuration import config as ewc_hub_config
from ewccli.utils import download_items


class DummyResponse:
    """Dummy response object for mocking requests.get."""

    def __init__(self, text: str = "dummy content", raise_exc: Exception = None):
        self.text = text
        self._raise_exc = raise_exc

    def raise_for_status(self):
        """Raise exception if configured, else succeed."""
        if self._raise_exc:
            raise self._raise_exc


@pytest.fixture
def temp_config(monkeypatch, tmp_path):
    """
    Fixture to redirect EWC config paths into a temporary directory.

    Args:
        monkeypatch: pytest fixture to patch module attributes.
        tmp_path: temporary path provided by pytest.

    Yields:
        Path: temporary config directory.
    """
    monkeypatch.setattr(ewc_hub_config, "EWC_CLI_BASE_PATH", tmp_path)
    monkeypatch.setattr(
        ewc_hub_config, "EWC_CLI_HUB_ITEMS_PATH", tmp_path / "items.yaml"
    )
    monkeypatch.setattr(
        ewc_hub_config, "EWC_CLI_HUB_ITEMS_URL", "https://fake-url/items.yaml"
    )
    yield tmp_path


def test_download_skips_if_file_exists_and_force_false(temp_config, monkeypatch):
    """
    Test that `download_items` does not re-download when file exists and force=False.
    """
    item_file = ewc_hub_config.EWC_CLI_HUB_ITEMS_PATH
    item_file.write_text("existing content")

    monkeypatch.setattr(
        "requests.get", lambda *a, **kw: pytest.fail("Should not be called")
    )

    download_items(force=False)

    assert item_file.read_text() == "existing content"


def test_download_redownloads_if_force_true(temp_config, monkeypatch):
    """
    Test that `download_items` redownloads when force=True even if file exists.
    """
    item_file = ewc_hub_config.EWC_CLI_HUB_ITEMS_PATH
    item_file.write_text("old content")

    monkeypatch.setattr("requests.get", lambda *a, **kw: DummyResponse("new content"))

    download_items(force=True)

    assert item_file.read_text() == "new content"


def test_download_creates_new_file(temp_config, monkeypatch):
    """
    Test that `download_items` downloads and creates the file when it does not exist.
    """
    monkeypatch.setattr(
        "requests.get", lambda *a, **kw: DummyResponse("downloaded data")
    )

    download_items(force=False)

    assert ewc_hub_config.EWC_CLI_HUB_ITEMS_PATH.read_text() == "downloaded data"


def test_download_handles_timeout(temp_config, monkeypatch, caplog):
    """
    Test that `download_items` logs an error when a timeout occurs.
    """

    def fake_get(*a, **kw):
        raise requests.Timeout()

    monkeypatch.setattr("requests.get", fake_get)

    download_items(force=True)

    assert "timed out" in caplog.text


def test_download_handles_request_exception(temp_config, monkeypatch, caplog):
    """
    Test that `download_items` logs an error when a RequestException occurs.
    """

    def fake_get(*a, **kw):
        raise requests.RequestException("connection error")

    monkeypatch.setattr("requests.get", fake_get)

    download_items(force=True)

    assert "connection error" in caplog.text



#!/usr/bin/env python
#
# Package Name: ewccli
# License: GPL-3.0-or-later
# Copyright (c) 2025 EUMETSAT, ECMWF for European Weather Cloud
# See the LICENSE file for more details


"""CLI EWC Hub: EWC Hub interaction items specific methods."""

import shutil
import sys
import json
import time
from pathlib import Path
from typing import Tuple, Optional

import requests
from openstack import connection

from ewccli.configuration import config as ewc_hub_config
from ewccli.utils import run_command_from_host
from ewccli.enums import Federee
from ewccli.backends.ansible.backend_ansible import AnsibleBackend
from ewccli.logger import get_logger

_LOGGER = get_logger(__name__)

ansible_backend = AnsibleBackend()

HUB_ENV_VARIABLES_MAP = {
    "os_network_name": {Federee.ECMWF.value: None, Federee.EUMETSAT.value: "private"},
    "os_subnet_name": {
        Federee.ECMWF.value: None,
        Federee.EUMETSAT.value: "private-subnet",
    },
    "os_subnet_cidr": {
        Federee.ECMWF.value: "192.168.1.0/24",
        Federee.EUMETSAT.value: "10.0.0.0/24",
    },
    "dns_domain": {
        Federee.ECMWF.value: None,
        Federee.EUMETSAT.value: None,
    },
}


def get_hub_item_env_variable_value(
    hub_item_env_variables_map: dict,
    federee: str,
    tenancy_name: str,
    variable_name: str,
    openstack_api: Optional[connection.Connection] = None,
) -> str:
    """
    Retrieve the value of a HUB_ENV_VARIABLES_MAP variable for a given federee.

    If the value is a string or list defined in HUB_ENV_VARIABLES_MAP, return it directly.
    If the value must be determined dynamically from the OpenStack client, query it.

    Args:
        hub_item_env_variables_map
        openstack_api: An authenticated OpenStack SDK client.
        federee (str): The federee key (e.g., Federee.ECMWF.value, Federee.EUMETSAT.value).
        tenancy_name (str): tenancy_name from config file created with ewc login.
        variable_name (str): The variable name to retrieve from HUB_ENV_VARIABLES_MAP.

    Returns:
        Any: The resolved value (string, list, etc.).

    Raises:
        KeyError: If the variable or federee entry does not exist.
        ValueError: If dynamic lookup fails.
    """
    os_network_name = "private"
    os_subnetwork_name = "private-subnet"

    # --- Handle dynamic lookups ---
    if variable_name == "os_network_name" and federee == Federee.ECMWF.value:
        if openstack_api:
            networks = list(openstack_api.network.networks())
            for net in networks:
                if "private" in net.name.lower():
                    os_network_name = net.name
            raise ValueError("No network containing 'private' found.")

    hub_item_env_variables_map["os_network_name"] = {
        Federee.ECMWF.value: os_network_name,
        Federee.EUMETSAT.value: "private",
    }

    if variable_name == "os_subnet_name" and federee == Federee.ECMWF.value:
        if openstack_api:
            networks = list(openstack_api.network.networks())
            for net in networks:
                if "private" in net.name.lower():
                    subnets = list(openstack_api.network.subnets(network_id=net.id))
                    if subnets:
                        os_subnetwork_name = subnets[0].name
            raise ValueError("No subnet found for network containing 'private'.")

    hub_item_env_variables_map["os_subnet_name"] = {
        Federee.ECMWF.value: os_subnetwork_name,
        Federee.EUMETSAT.value: "private-subnet",
    }

    if variable_name == "dns_domain":
        dns_domain = (
            f"{tenancy_name}.{ewc_hub_config.FEDEREE_DNS_MAPPING[federee]}.ewcloud.host"
        )
        hub_item_env_variables_map["dns_domain"] = {
            federee: dns_domain,
        }

    # If not dynamic, return directly
    return hub_item_env_variables_map[variable_name][federee]


def check_github_repo_accessible(source: str) -> bool:
    """
    Check if a GitHub repository exists and is publicly accessible.

    Args:
        source (str): The full URL of the GitHub repository.

    Returns:
        bool: True if the repository is accessible, False otherwise.
    """
    # Remove trailing .git if present
    if source.endswith(".git"):
        source = source[:-4]

    # Normalize trailing slash
    source = source.rstrip("/")

    # Build API URL
    api_url = source.replace("https://github.com/", "https://api.github.com/repos/")

    try:
        response = requests.get(api_url, timeout=5)
        if response.status_code == 200:
            _LOGGER.info(f"✅ Repository is accessible: [blue]{source}[/blue]")
            return True
        elif response.status_code == 404:
            _LOGGER.error(f"❌ Repository not found: [red]{source}[/red]")
        else:
            _LOGGER.error(
                f"⚠️ Unexpected response ({response.status_code}) when checking: {source}"
            )
    except requests.RequestException as e:
        _LOGGER.error(f"🚨 Network error while accessing repository: {e}")

    return False


def git_clone_item(
    source: str,
    repo_name: str,
    command_path: str,
    dry_run: bool = False,
    force: bool = False,
):
    """Git clone item."""
    ########################################################################
    # Prepare input for items
    ########################################################################
    _LOGGER.info("Git clone item...")
    ########################################################################
    # Git clone item to the correct path
    ########################################################################

    # Define the directory path for outputs
    dir_path = Path(command_path)

    # Create the directory if it doesn't exist
    dir_path.mkdir(parents=True, exist_ok=True)

    # main_file_name = os.path.basename(source)

    repo_path = Path(f"{command_path}/{repo_name}")
    # file_name_path = Path(f"{repo_path}/{main_file_name}")

    if force:
        # Delete the folder and all its contents
        shutil.rmtree(repo_path)

    if not check_github_repo_accessible(source):
        _LOGGER.error("The repository is not accessible or does not exist.")
        sys.exit(1)

    if repo_path.exists() and not force:
        return (
            0,
            f"📁 Main Repository {repo_name} already exists at {command_path}. Skipping git clone.",
        )

    _LOGGER.info(
        f"⬇️ Starting to clone the repository '{repo_name}' into {command_path}..."
    )
    git_command = [
        f"""[ -d "$(basename '{source}' .git)/.git" ] || git clone '{source}' """
    ]
    # Get Git Repository URL
    return_code, message = run_command_from_host(
        description="git clone repository",
        command=git_command,
        cwd=command_path,
        dry_run=dry_run,
    )

    return return_code, message


def run_ansible_item(
    item: str,
    item_inputs: Optional[dict],
    server_name: str,
    ip_machine: str,
    username: str,
    main_file_path: str,
    requirements_file_path: str,
    working_directory_path: str,
    ssh_private_key_path: str,
    dry_run: bool = False,
):
    """Run item based on Ansible Playbook."""
    if dry_run:
        return 0, "Dry run. No actions"

    # Install roles
    ansible_backend.install_ansible_roles(
        requirements_path=requirements_file_path, dry_run=dry_run
    )

    # Create inventory file
    if not ip_machine:
        machine_ips = [""]
    else:
        machine_ips = [ip_machine]

    hosts_file_name = f"hosts-{item}.ini"
    hosts_file_path = f"{working_directory_path}/{hosts_file_name}"
    _LOGGER.debug(f"Saving {hosts_file_name} file to {hosts_file_path}")

    inventory_content = f"[{server_name}]\n"
    inventory_content += "\n".join(machine_ips) + "\n\n"

    _LOGGER.debug(f"Inventory content {inventory_content}")

    with open(hosts_file_path, "w") as f:
        f.write(inventory_content)

    # ansible_command = [
    #     f"ANSIBLE_HOST_KEY_CHECKING=False ansible-playbook -i {hosts_file_path} -u {username}"
    #     f" --private-key {ssh_private_key_path} {main_file_path}"
    # ]
    # return_code = ansible_backend.run_ansible(
    #     description="run ansible",
    #     command=ansible_command,
    #     cwd=f"{working_directory_path}",
    #     dry_run=dry_run
    # )

    env = {
        # "ANSIBLE_PRIVATE_KEY_FILE": ssh_private_key_path,
        "ANSIBLE_HOST_KEY_CHECKING": "False",  # Optional, disables host key prompt,
        "ANSIBLE_PORT": "2222",  # <- Set SSH port globally
        "ANSIBLE_PYTHON_INTERPRETER": "/usr/bin/python3",
        # "ANSIBLE_SSH_ARGS": "-o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no",
    }

    ansible_command = [
        f"ansible-playbook -i {hosts_file_path} -u {username}"
        f" --private-key {ssh_private_key_path} {main_file_path}"
    ]

    _LOGGER.info(f"Deploying Ansible Playbook item {item}...")
    _LOGGER.info("⏳ This could take a few minutes, grab a beverage meanwhile...")

    # Sleep some time to wait to start.
    time.sleep(15)

    if item_inputs:
        extra_vars = json.dumps(item_inputs)
    else:
        extra_vars = ""

    max_attempts = 5
    delay_seconds = 10  # wait between attempts

    for attempt in range(1, max_attempts + 1):
        _LOGGER.info(f"Running attempt {attempt}/{max_attempts}...")

        return_code = ansible_backend.run_ansible_live(
            working_directory_path=f"{working_directory_path}",
            description=ansible_command[0],
            host=server_name,
            cmdline=[
                "ansible-playbook",
                "-i",
                hosts_file_path,
                "-u",
                username,
                "--private-key",
                ssh_private_key_path,
                main_file_path,
            ],
            extra_vars=extra_vars,
            env=env,
        )

        if return_code == 0:
            # Success
            _LOGGER.info(f"Attempt {attempt}/{max_attempts} succeeded.")
            return return_code
        else:
            _LOGGER.warning(f"Attempt {attempt}/{max_attempts} failed")
            if attempt < max_attempts:
                _LOGGER.info(f"Retrying in {delay_seconds} seconds...")
                time.sleep(delay_seconds)
            else:
                _LOGGER.error(
                    f"All attempts failed. EWC CLI could not install {item} Ansible Playbook item."
                )
                return_code = 1

    return return_code


def run_post_ansible_operations(
    item: str,
    command_path: str,
    repo_name: str,
    server_name: str,
    internal_ip_machine: str,
):
    """Run post ansible operation if something goes wrong."""
    hosts_file_name = f"hosts-{item}.ini"
    hosts_file_path = f"{command_path}/{repo_name}/{hosts_file_name}"

    inventory_content = f"[{server_name}]\n"
    inventory_content += "\n".join([internal_ip_machine]) + "\n\n"

    _LOGGER.debug(f"Post run inventory content {inventory_content}")

    with open(hosts_file_path, "w") as f:
        f.write(inventory_content)


def run_ansible_playbook_item(
    item: str,
    server_name: str,
    username: str,
    main_file_path: str,
    requirements_file_path: str,
    working_directory_path: str,
    ip_machine: str,
    ssh_private_key_path: str,
    item_inputs: Optional[dict],
    dry_run: bool = False,
) -> Tuple[int, str]:
    """Deploy Ansible item."""
    ansible_return_code = run_ansible_item(
        item=item,
        item_inputs=item_inputs,
        server_name=server_name,
        ip_machine=ip_machine,
        username=username,
        main_file_path=main_file_path,
        requirements_file_path=requirements_file_path,
        working_directory_path=working_directory_path,
        ssh_private_key_path=ssh_private_key_path,
        dry_run=dry_run,
    )

    if ansible_return_code != 0:
        return ansible_return_code, "⚠️ Ansible execution failed. Check errors above."

    return 0, "Ansible run successfully"



#!/usr/bin/env python
#
# Package Name: ewccli
# License: GPL-3.0-or-later
# Copyright (c) 2025 EUMETSAT, ECMWF for European Weather Cloud
# See the LICENSE file for more details


"""Tests for EWC hub deploy server command methods."""

from typing import Optional

import pytest
from pydantic import BaseModel
from unittest.mock import MagicMock
from datetime import datetime, timedelta

from ewccli.tests.ewccli_base_test import ServerInfo
from ewccli.tests.ewccli_base_test import Address
from ewccli.configuration import EWCCLIConfiguration as ewc_hub_config
from ewccli.enums import Federee
from ewccli.backends.openstack.backend_ostack import OpenstackBackend
from ewccli.commands.commons_infra import resolve_machine_ip
from ewccli.commands.commons_infra import normalize_os_image

# ---------------------------------------------------------------------------
# Fake image using Pydantic for strict validation
# ---------------------------------------------------------------------------

class FakeImage(BaseModel):
    name: str
    created_at: datetime


@pytest.fixture
def conn():
    """Mock OpenStack connection with compute.images()."""
    mock_conn = MagicMock()
    mock_conn.compute.images = MagicMock()
    return mock_conn


@pytest.fixture
def finder():
    """Use find_latest_image as unbound method, no backend instance needed."""
    return OpenstackBackend.find_latest_image


# ---------------------------------------------------------------------------
# CPU: Rocky-8
# ---------------------------------------------------------------------------

def test_find_latest_rocky8_cpu(finder, conn, monkeypatch):
    now = datetime.utcnow()
    img_old = FakeImage(name="Rocky-8.9-20250101010101", created_at=now - timedelta(days=10))
    img_new = FakeImage(name="Rocky-8.9-20250202020202", created_at=now)

    conn.compute.images.return_value = [img_old, img_new]

    # Correct target to patch!
    monkeypatch.setattr(
        "ewccli.configuration.EWCCLIConfiguration.EWC_CLI_CPU_IMAGES",
        {"Rocky-8", "Rocky-9", "Ubuntu-22.04", "Ubuntu-24.04"},
    )

    result = finder(None, conn, "Rocky-8")
    assert result == img_new


# ---------------------------------------------------------------------------
# CPU: Ubuntu 22.04
# ---------------------------------------------------------------------------

def test_find_latest_ubuntu_2204_cpu(finder, conn, monkeypatch):
    now = datetime.utcnow()
    img1 = FakeImage(name="Ubuntu-22.04-20250101010101", created_at=now - timedelta(days=5))
    img2 = FakeImage(name="Ubuntu-22.04-20250303030303", created_at=now)

    conn.compute.images.return_value = [img1, img2]

    monkeypatch.setattr(
        "ewccli.configuration.EWCCLIConfiguration.EWC_CLI_CPU_IMAGES",
        {"Rocky-8", "Rocky-9", "Ubuntu-22.04", "Ubuntu-24.04"},
    )

    result = finder(None, conn, "Ubuntu-22.04")
    assert result == img2


# ---------------------------------------------------------------------------
# GPU: Rocky
# ---------------------------------------------------------------------------

def test_find_latest_rocky_gpu(finder, conn, monkeypatch):
    now = datetime.utcnow()
    img1 = FakeImage(name="Rocky-9.6-GPU-20250101010101", created_at=now - timedelta(days=3))
    img2 = FakeImage(name="Rocky-9.6-GPU-20250303030303", created_at=now)

    conn.compute.images.return_value = [img1, img2]

    monkeypatch.setattr(
        "ewccli.configuration.EWCCLIConfiguration.EWC_CLI_CPU_IMAGES",
        {"Rocky-8", "Rocky-9", "Ubuntu-22.04", "Ubuntu-24.04"},
    )

    result = finder(None, conn, "Rocky-9.6-GPU")
    assert result == img2


# ---------------------------------------------------------------------------
# GPU: Ubuntu
# ---------------------------------------------------------------------------

def test_find_latest_ubuntu_gpu(finder, conn, monkeypatch):
    now = datetime.utcnow()
    img1 = FakeImage(name="Ubuntu 22.04 NVIDIA_AI", created_at=now - timedelta(days=1))
    img2 = FakeImage(name="Ubuntu 22.04 NVIDIA_AI", created_at=now)

    conn.compute.images.return_value = [img1, img2]

    # GPU mapping
    monkeypatch.setattr(
        "ewccli.configuration.EWCCLIConfiguration.EWC_CLI_OS_GPU_IMAGES_SITE_MAP",
        {"EUMETSAT": "Ubuntu 22.04 NVIDIA_AI"},
    )

    monkeypatch.setattr(
        "ewccli.configuration.EWCCLIConfiguration.EWC_CLI_CPU_IMAGES",
        {"Rocky-8", "Rocky-9", "Ubuntu-22.04", "Ubuntu-24.04"},
    )

    result = finder(None, conn, "Ubuntu 22.04 NVIDIA_AI")
    assert result == img2


# ---------------------------------------------------------------------------
# No match
# ---------------------------------------------------------------------------

def test_no_matching_images(finder, conn, monkeypatch):
    conn.compute.images.return_value = [
        FakeImage(name="UnrelatedImage", created_at=datetime.utcnow())
    ]

    monkeypatch.setattr(
        "ewccli.configuration.EWCCLIConfiguration.EWC_CLI_CPU_IMAGES",
        {"Rocky-8", "Rocky-9", "Ubuntu-22.04", "Ubuntu-24.04"},
    )

    assert finder(None, conn, "Rocky-8") is None


# Pydantic models

class IPResult(BaseModel):
    """
    Pydantic model representing the result of resolving a machine's IPs.

    Attributes:
        internal_ip_machine (Optional[str]): The internal/private IP of the machine.
        external_ip_machine (Optional[str]): The external/public IP of the machine.
    """

    internal_ip_machine: Optional[str]
    external_ip_machine: Optional[str]


@pytest.mark.parametrize(
    "federee,server_info,expected_status,expected_result",
    [
        # EUMETSAT with both fixed and floating IPs
        (
            Federee.EUMETSAT.value,
            ServerInfo(
                id="server-001",
                name="eumetsat-server",
                flavor={"name": "small"},
                key_name="key1",
                status="ACTIVE",
                addresses={
                    "private": [
                        Address(addr="10.0.0.5", **{"OS-EXT-IPS:type": "fixed"}),
                        Address(
                            addr="192.168.1.100", **{"OS-EXT-IPS:type": "floating"}
                        ),
                    ]
                },
                security_groups=[],
            ),
            0,
            IPResult(
                internal_ip_machine="10.0.0.5", external_ip_machine="192.168.1.100"
            ),
        ),
        # ECMWF with default private/external IP
        (
            Federee.ECMWF.value,
            ServerInfo(
                id="server-002",
                name="ecmwf-server",
                flavor={"name": "medium"},
                key_name="key2",
                status="ACTIVE",
                addresses={
                    "private-1": [
                        Address(addr="10.1.1.5"),
                        Address(addr="136.10.10.10"),
                    ],
                    "external-net": [Address(addr="200.100.50.1")],
                },
                security_groups=[],
            ),
            0,
            IPResult(
                internal_ip_machine="10.1.1.5", external_ip_machine="136.10.10.10"
            ),
        ),
        # ECMWF specifying external network
        (
            Federee.ECMWF.value,
            ServerInfo(
                id="server-003",
                name="ecmwf-server-ext",
                flavor={"name": "medium"},
                key_name="key3",
                status="ACTIVE",
                addresses={
                    "private-1": [Address(addr="10.1.1.5")],
                    "external-internet": [Address(addr="200.100.50.1")],
                },
                security_groups=[],
            ),
            0,
            IPResult(
                internal_ip_machine="10.1.1.5", external_ip_machine="200.100.50.1"
            ),
        ),
        # Missing addresses (should fail)
        (
            Federee.ECMWF.value,
            ServerInfo(
                id="server-004",
                name="ecmwf-server-missing",
                flavor={"name": "small"},
                key_name="key4",
                status="ACTIVE",
                addresses={},
                security_groups=[],
            ),
            1,
            None,
        ),
    ],
)
def test_resolve_machine_ip(federee, server_info, expected_status, expected_result):
    """
    Test the resolve_machine_ip function for multiple scenarios.

    Args:
        federee (str): The federee (EUMETSAT or ECMWF) for which to resolve IPs.
        server_info (ServerInfo): Server information object with addresses.
        expected_status (int): Expected status code returned by the function (0 for success, 1 for error).
        expected_result (Optional[IPResult]): Expected resolved IPs. None for error cases.

    Asserts:
        - The returned status code matches the expected status.
        - The resolved internal and external IPs match the expected result if provided.
        - The result is None in error cases.
    """
    status, message, result = resolve_machine_ip(
        federee,
        server_info.model_dump(by_alias=True, exclude_none=False),
    )
    assert status == expected_status

    if expected_result is not None:
        ip_result = IPResult(**result)
        assert ip_result == expected_result
    else:
        assert result is None


# ======================================================================
# Tests
# ======================================================================

@pytest.fixture(autouse=True)
def clean_config(monkeypatch):
    monkeypatch.setattr(
        ewc_hub_config,
        "EWC_CLI_CPU_IMAGES",
        set()
    )
    monkeypatch.setattr(
        ewc_hub_config,
        "EWC_CLI_OS_GPU_IMAGES_SITE_MAP",
        {
            "ECMWF": "Rocky-9.6-GPU",
            "EUMETSAT": "Ubuntu-22.04-NVIDIA_AI",
        }
    )


# ----------------------- CPU Tests -----------------------

def test_exact_cpu_match(monkeypatch):
    monkeypatch.setattr(ewc_hub_config, "EWC_CLI_CPU_IMAGES", {"Rocky-8", "Ubuntu-22.04"})

    normalized, exact = normalize_os_image("Rocky-8", "ECMWF")
    assert normalized == "Rocky-8"
    assert exact is True


def test_normalize_rocky_timestamp(monkeypatch):
    monkeypatch.setattr(ewc_hub_config, "EWC_CLI_CPU_IMAGES", {"Rocky-9"})

    normalized, exact = normalize_os_image("Rocky-9.6-20251107141503", "ECMWF")
    assert normalized == "Rocky-9"
    assert exact is False


def test_normalize_ubuntu_timestamp(monkeypatch):
    monkeypatch.setattr(ewc_hub_config, "EWC_CLI_CPU_IMAGES", {"Ubuntu-24.04"})

    normalized, exact = normalize_os_image("Ubuntu-24.04-20251107141503", "EUMETSAT")
    assert normalized == "Ubuntu-24.04"
    assert exact is False


# ----------------------- EUMETSAT GPU -----------------------

def test_eumetsat_exact_gpu(monkeypatch):
    monkeypatch.setattr(
        ewc_hub_config,
        "EWC_CLI_OS_GPU_IMAGES_SITE_MAP",
        {"EUMETSAT": "Ubuntu-22.04-NVIDIA_AI"}
    )

    normalized, exact = normalize_os_image("Ubuntu-22.04-NVIDIA_AI", "EUMETSAT")
    assert normalized == "Ubuntu-22.04-NVIDIA_AI"
    assert exact is False


def test_eumetsat_translate_generic_gpu(monkeypatch):
    monkeypatch.setattr(
        ewc_hub_config,
        "EWC_CLI_OS_GPU_IMAGES_SITE_MAP",
        {"EUMETSAT": "Ubuntu-22.04-NVIDIA_AI"}
    )

    normalized, exact = normalize_os_image("Ubuntu-22.04-GPU", "EUMETSAT")
    assert normalized == "Ubuntu-22.04-NVIDIA_AI"
    assert exact is True


# ----------------------- ECMWF GPU -----------------------

def test_ecmwf_exact_gpu(monkeypatch):
    monkeypatch.setattr(
        ewc_hub_config,
        "EWC_CLI_OS_GPU_IMAGES_SITE_MAP",
        {"ECMWF": "Rocky-9-GPU"}
    )

    normalized, exact = normalize_os_image("Rocky-9-GPU", "ECMWF")
    assert normalized == "Rocky-9-GPU"
    assert exact is True


def test_ecmwf_timestamp_gpu(monkeypatch):
    monkeypatch.setattr(
        ewc_hub_config,
        "EWC_CLI_OS_GPU_IMAGES_SITE_MAP",
        {"ECMWF": "Rocky-9-GPU"}
    )

    normalized, exact = normalize_os_image("Rocky-9.6-GPU-20250101010101", "ECMWF")
    assert normalized == "Rocky-9-GPU"
    assert exact is False


# ----------------------- Unknown -----------------------

def test_unknown_image(monkeypatch):
    monkeypatch.setattr(ewc_hub_config, "EWC_CLI_CPU_IMAGES", {"Rocky-8"})

    normalized, exact = normalize_os_image("NotAnImage", "EUMETSAT")
    assert normalized is None
    assert exact is False




#!/usr/bin/env python
#
# Package Name: ewccli
# License: GPL-3.0-or-later
# Copyright (c) 2025 EUMETSAT, ECMWF for European Weather Cloud
# See the LICENSE file for more details


"""European Weather Cloud (EWC) CLI."""

from importlib.metadata import version, PackageNotFoundError
from typing import Optional
import rich_click as click

from ewccli import __title__
from ewccli.commands.login_command import init_command, init_options

# Multiple backends
from ewccli.commands.hub.hub_command import ewc_hub_command

# Openstack backend
from ewccli.commands.infra_command import ewc_infra_command

# Crossplane backend
# from ewccli.commands.k8s_command import ewc_k8s_command
# from ewccli.commands.dns_command import ewc_dns_command
# from ewccli.commands.s3_command import ewc_s3_command

from ewccli.configuration import config as ewc_hub_config


CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])


@click.group(context_settings=CONTEXT_SETTINGS)
def cli():
    """European Weather Cloud (EWC) CLI."""
    pass


@cli.command(name="login", help="Initialize configuration for EWC CLI.")
@init_options
def init(
    application_credential_id: str,
    application_credential_secret: str,
    ssh_public_key_path: str,
    ssh_private_key_path: str,
    tenant_name: str,
    federee: str,
    profile: Optional[str] = None,
    # token: str,
):
    """Login command."""
    init_command(
        application_credential_id=application_credential_id,
        application_credential_secret=application_credential_secret,
        ssh_public_key_path=ssh_public_key_path,
        ssh_private_key_path=ssh_private_key_path,
        tenant_name=tenant_name,
        federee=federee,
        profile=profile,
        # token=token,
    )


def get_version():
    """
    Return the version of the installed package.

    This function first attempts to retrieve the version number using
    `importlib.metadata.version()`, which works when the package is installed
    normally (e.g., via pip). If the package is not installed and a
    `PackageNotFoundError` is raised, the version is retrieved directly from
    the package's `__version__` attribute, which supports local development
    without installation.

    Returns:
        str: The version string of the package.
    """
    try:
        return version(__title__)  # the name in pyproject.toml / setup.py
    except PackageNotFoundError:
        # fallback to direct import if running in source mode
        from ewccli import __version__

        return __version__


@cli.command(name="version", help="Show EWC CLI version.")
def version_cmd():
    """
    Display the version of the EWC CLI.

    This command reads the installed package version using `get_version()`
    and prints it to the terminal. It is useful for debugging, user support,
    and verifying correct installation of the CLI.
    """
    click.echo(get_version())


# Register subcommands
cli.add_command(ewc_hub_command)
cli.add_command(ewc_infra_command)
# cli.add_command(ewc_k8s_command)
# cli.add_command(ewc_dns_command)
# cli.add_command(ewc_s3_command)


if __name__ == "__main__":
    cli(prog_name=f"{ewc_hub_config.EWC_CLI_NAME}")



#!/usr/bin/env python
#
# Package Name: ewccli
# License: GPL-3.0-or-later
# Copyright (c) 2025 EUMETSAT, ECMWF for European Weather Cloud
# See the LICENSE file for more details


"""Tests for EWC commands common methods."""

from unittest.mock import MagicMock
import pytest
from pydantic import BaseModel
from datetime import datetime

from ewccli.tests.ewccli_base_test import SecurityGroup
from ewccli.tests.ewccli_base_test import ServerInfo

from ewccli.enums import Federee
from ewccli.configuration import EWCCLIConfiguration as ewc_hub_config
from ewccli.backends.openstack.backend_ostack import OpenstackBackend
from ewccli.commands.commons_infra import get_deployed_server_info
from ewccli.commands.commons_infra import resolve_image_and_flavor
from ewccli.commands.commons_infra import normalize_os_image

# --- Fake OpenstackBackend for testing -------------------------------------
class FakeImage:
    def __init__(self, name: str):
        self.name = name
        self.created_at = datetime.utcnow()

# --- Mock backend completely -------------------------------------------------
class FakeOpenstackBackend:
    def __init__(self, *args, **kwargs):
        pass  # skip real OpenStack connection

    def find_latest_image(self, conn, image_prefix: str):
        # Return a fake "latest image" object
        class FakeImage:
            def __init__(self, name):
                self.name = name

        # simulate the "latest image" based on prefix
        if image_prefix.startswith("Ubuntu-22.04"):
            return FakeImage("Ubuntu-22.04-20250202020202")
        elif image_prefix.startswith("Ubuntu-24.04"):
            return FakeImage("Ubuntu-24.04-20250202020202")
        elif image_prefix.startswith("Rocky-9"):
            return FakeImage("Rocky-9-20250202020202")
        elif image_prefix.startswith("Rocky-8"):
            return FakeImage("Rocky-8-20250202020202")
        elif image_prefix.startswith("Rocky-9-GPU"):
            return FakeImage("Rocky-9.6-GPU-20250202020202")
        elif image_prefix.startswith("Ubuntu 22.04 NVIDIA_AI"):
            return FakeImage("Ubuntu 22.04 NVIDIA_AI")
        else:
            return None

# --- Fixtures ---------------------------------------------------------------
@pytest.fixture
def backend():
    return FakeOpenstackBackend()

@pytest.fixture
def conn():
    return MagicMock()  # mock OpenStack connection

# --- Tests -----------------------------------------------------------------
def test_resolve_cpu_defaults(conn, backend):
    code, msg, result = resolve_image_and_flavor(conn, backend, federee="EUMETSAT", is_gpu=False)
    assert code == 0
    assert result["normalized_image_name"] in ewc_hub_config.EWC_CLI_CPU_IMAGES
    assert result["flavour_name"] == ewc_hub_config.DEFAULT_CPU_FLAVOURS_MAP["EUMETSAT"]

def test_resolve_eumetsat_gpu_defaults(conn, backend):
    code, msg, result = resolve_image_and_flavor(conn, backend, federee="EUMETSAT", is_gpu=True)
    assert code == 0
    assert result["normalized_image_name"] == ewc_hub_config.EWC_CLI_OS_GPU_IMAGES_SITE_MAP["EUMETSAT"]
    assert result["flavour_name"] == ewc_hub_config.DEFAULT_GPU_FLAVOURS_MAP["EUMETSAT"]

def test_resolve_ecmwf_gpu_defaults(conn, backend):
    code, msg, result = resolve_image_and_flavor(conn, backend, federee="ECMWF", is_gpu=True)
    assert code == 0
    assert result["normalized_image_name"] == ewc_hub_config.EWC_CLI_OS_GPU_IMAGES_SITE_MAP["ECMWF"]
    assert result["flavour_name"] == ewc_hub_config.DEFAULT_GPU_FLAVOURS_MAP["ECMWF"]

def test_resolve_specific_image(conn, backend):
    code, msg, result = resolve_image_and_flavor(
        conn,
        backend,
        federee="EUMETSAT",
        image_name="Ubuntu-22.04-20250202020202",
        flavour_name="vm.a6000.1"
    )
    assert code == 0

    # normalized image name is short version
    normalized, _ = normalize_os_image("Ubuntu-22.04-20250202020202", "EUMETSAT")
    assert result["normalized_image_name"] == normalized
    assert result["flavour_name"] == "vm.a6000.1"

def test_resolve_unknown_image(conn, backend):
    code, msg, result = resolve_image_and_flavor(
        conn,
        backend,
        federee="EUMETSAT",
        image_name="Unknown-OS-1234"
    )
    assert code == 1
    assert "Unsupported OS image" in msg

#################################################################################################
# --- Tests ---
def test_get_deployed_server_info_eumetsat_private_and_manila():
    """Test EUMETSAT federee with private and manila-network addresses."""
    server = ServerInfo(
        id="02406c28-a84a-4829-bd6b-5562cd6eae8c",
        name="test-vm",
        flavor={"original_name": "m1.small"},
        key_name="my-key",
        status="ACTIVE",
        addresses={
            "private": [{"addr": "10.0.0.5", "OS-EXT-IPS:type": "fixed"}],
            "manila-network": [{"addr": "192.168.1.5"}],
        },
        security_groups=[SecurityGroup(name="ssh")],
    )

    vm_info = get_deployed_server_info(
        Federee.EUMETSAT.value,
        server.model_dump(by_alias=True),
        image_name="ubuntu-20.04",
    )

    assert vm_info["id"] == "02406c28-a84a-4829-bd6b-5562cd6eae8c"
    assert vm_info["flavor"] == "m1.small"
    assert vm_info["networks"]["network-private-fixed"] == "10.0.0.5"
    assert vm_info["networks"]["sfs-manila-network"] == "192.168.1.5"
    assert vm_info["security-groups"] == ["ssh"]
    assert vm_info["image"] == "ubuntu-20.04"


def test_get_deployed_server_info_ecmwf_multiple_networks():
    """Test ECMWF federee with multiple networks."""
    server = ServerInfo(
        id="02406c28-b84a-4829-bd6b-5562cd6eae8c",
        name="ecmwf-vm",
        flavor={"original_name": "m2.medium"},
        key_name="ecmwf-key",
        status="BUILD",
        addresses={
            "net1": [{"addr": "172.16.0.10"}, {"addr": "172.16.0.11"}],
            "net2": [{"addr": "10.10.10.5"}],
        },
        security_groups=[SecurityGroup(name="sec1"), SecurityGroup(name="sec2")],
    )

    vm_info = get_deployed_server_info(Federee.ECMWF.value, server.model_dump())

    assert vm_info["id"] == "02406c28-b84a-4829-bd6b-5562cd6eae8c"
    assert vm_info["flavor"] == "m2.medium"
    assert vm_info["networks"]["network-net1"] == ["172.16.0.10", "172.16.0.11"]
    assert vm_info["networks"]["network-net2"] == ["10.10.10.5"]
    assert set(vm_info["security-groups"]) == {"sec1", "sec2"}


def test_get_deployed_server_info_no_addresses():
    """Test server with no addresses."""
    server = ServerInfo(
        id="02406c28-a84a-4829-bd6b-5562cd6eae8c",
        name="no-address-vm",
        flavor={"original_name": "tiny"},
        key_name="none",
        status="SHUTOFF",
        addresses=None,
        security_groups=[],
    )

    vm_info = get_deployed_server_info(Federee.EUMETSAT.value, server.model_dump())

    assert vm_info["networks"] == {}
    assert vm_info["security-groups"] == []



#!/usr/bin/env python
#
# Package Name: ewccli
# License: GPL-3.0-or-later
# Copyright (c) 2025 EUMETSAT, ECMWF for European Weather Cloud
# See the LICENSE file for more details

"""S3 CRD classes."""

from dataclasses import dataclass, field
from typing import List, Optional, Dict

from ewccli.backends.kubernetes.CRDtemplates.commoncrd import GroupVersionResource


@dataclass
class BucketSpec:
    """
    Specification for a Bucket custom resource.

    Attributes:
        siteName (str): Name of the site where this bucket will be deployed.
        bucketName (str): Name of the S3 bucket.
        owner (str): Access ID of the S3 credentials (owner of the bucket).
        writeAccessIds (Optional[List[str]]): IDs that have write access to the bucket.
        writeAccessRefsIds (Optional[List[str]]): References to IDs with write access.
        readAccessIds (Optional[List[str]]): IDs that have read access to the bucket.
        readAccessRefsIds (Optional[List[str]]): References to IDs with read access.
        georedundancy (Optional[dict]): Georedundancy configuration (enabled flag).
    """

    siteName: str
    bucketName: str
    owner: str
    writeAccessIds: Optional[List[str]] = None
    writeAccessRefsIds: Optional[List[str]] = None
    readAccessIds: Optional[List[str]] = None
    readAccessRefsIds: Optional[List[str]] = None
    georedundancy: Optional[Dict[str, bool]] = None


@dataclass
class Metadata:
    """
    Metadata for the custom resource.

    Attributes:
        name (str): Resource name (must match bucketName for S3 bucket CRs).
        namespace (str): Kubernetes namespace.
        labels (Optional[Dict]): Optional labels for resource metadata.
    """

    name: str
    namespace: str
    labels: Optional[Dict[str, str]] = field(default_factory=dict)


@dataclass
class BucketConfig:
    """
    Represents the full Bucket custom resource definition.

    Attributes:
        apiVersion (str): API version of the custom resource.
        kind (str): Kind of the resource (e.g., "Bucket").
        metadata (Metadata): Resource metadata.
        spec (BucketSpec): Bucket specification.
    """

    apiVersion: str
    kind: str
    metadata: Metadata
    spec: BucketSpec


BucketGVR = GroupVersionResource(
    group="s3.europeanweather.cloud", version="v1alpha1", resource="buckets"
)



#!/usr/bin/env python
#
# Package Name: ewccli
# License: GPL-3.0-or-later
# Copyright (c) 2025 EUMETSAT, ECMWF for European Weather Cloud
# See the LICENSE file for more details


"""Utils."""

import os
import base64
import sys
import subprocess
from pathlib import Path
import secrets
import string
from datetime import datetime, timezone
from typing import Optional, Tuple, IO, List, Dict

import requests
from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.backends import default_backend

from configparser import ConfigParser

import rich_click as click
from click import ClickException

from ewccli.configuration import config as ewc_hub_config
from ewccli.logger import get_logger

_LOGGER = get_logger(__name__)


def _resolve_profile(
    profile: Optional[str] = None,
    federee: Optional[str] = None,
    tenant_name: Optional[str] = None,
) -> str:
    """Return explicit profile or auto-generate one using federee-tenant."""
    if profile is not None:
        return profile

    if not federee or not tenant_name:
        click.secho(
            "❌ Either 'profile' must be provided or both 'federee' and 'tenant_name'.",
            fg="red",
            bold=True,
        )
        raise click.Abort()

    return f"{federee.lower()}-{tenant_name.lower()}"


def save_default_login_profile(
    federee: str,
    tenant_name: str,
    ssh_private_key_path_to_save: str,
    ssh_public_key_path_to_save: str,
    application_credential_id: Optional[str] = None,
    application_credential_secret: Optional[str] = None,
    region: Optional[str] = None,
    token: Optional[str] = None,
    profiles_file_path: Path = ewc_hub_config.EWC_CLI_PROFILES_PATH,
) -> None:
    """
    Save the default login profile to EWC_CLI_PROFILES_PATH only if it does not exist.
    If it already exists, do nothing (skip).

    Uses ewc_hub_config.EWC_CLI_DEFAULT_PROFILE_NAME as the profile name.
    """
    resolved_profile = _resolve_profile(
        profile=ewc_hub_config.EWC_CLI_DEFAULT_PROFILE_NAME,
    )

    cfg = ConfigParser()
    cfg.read(profiles_file_path)

    # Skip saving if the default profile already exists
    if resolved_profile in cfg:
        return

    # Save profile (reusing the unified save_cli_profile logic)

    save_cli_profile(
        federee=federee,
        tenant_name=tenant_name,
        ssh_private_key_path_to_save=ssh_private_key_path_to_save,
        ssh_public_key_path_to_save=ssh_public_key_path_to_save,
        profile=resolved_profile,
        token=token,
        application_credential_id=application_credential_id,
        application_credential_secret=application_credential_secret,
        region=region,
    )


def save_cli_profile(
    federee: str,
    tenant_name: str,
    ssh_private_key_path_to_save: str,
    ssh_public_key_path_to_save: str,
    profile: Optional[str] = None,
    token: Optional[str] = None,
    application_credential_id: Optional[str] = None,
    application_credential_secret: Optional[str] = None,
    region: Optional[str] = None,
    profiles_file_path: Path = ewc_hub_config.EWC_CLI_PROFILES_PATH,
) -> None:
    """
    Save all profile data (config + credentials) into a single profiles file.

    Parameters
    ----------
    federee : str
        Federee name.
    tenant_name : str
        Tenant name.
    ssh_private_key_path_to_save: str
        SSH private key path
    ssh_public_key_path_to_save: str
        SSH public key path
    profile : str, optional
        Explicit profile name. If None, auto-generated using federee-tenant.
    token : str, optional
        Authentication token.
    application_credential_id : str, optional
        Application credential ID.
    application_credential_secret : str, optional
        Application credential secret.
    region : str, optional
        Region for the profile.
    """
    resolved_profile = _resolve_profile(profile, federee, tenant_name)
    cfg = ConfigParser()
    cfg.read(profiles_file_path)

    # Fail if profile exists
    if resolved_profile in cfg:
        click.secho(
            f"❌ Profile '{resolved_profile}' already exists in {profiles_file_path}",
            fg="red",
            bold=True,
        )
        click.secho(
            "Use a different profile name or delete the existing profile first.",
            fg="yellow",
        )
        raise click.Abort()

    # --- Save profile data
    cfg[resolved_profile] = {}

    # Non-sensitive
    cfg[resolved_profile]["federee"] = federee
    cfg[resolved_profile]["tenant_name"] = tenant_name
    cfg[resolved_profile]["ssh_public_key_path"] = ssh_public_key_path_to_save
    cfg[resolved_profile]["ssh_private_key_path"] = ssh_private_key_path_to_save

    if region:
        cfg[resolved_profile]["region"] = region

    # Sensitive
    if token:
        cfg[resolved_profile]["token"] = token

    if application_credential_id:
        cfg[resolved_profile]["application_credential_id"] = application_credential_id

    if application_credential_secret:
        cfg[resolved_profile][
            "application_credential_secret"
        ] = application_credential_secret

    os.makedirs(os.path.dirname(profiles_file_path), exist_ok=True)
    with open(profiles_file_path, "w") as f:
        cfg.write(f)


def load_cli_profile(
    profile: Optional[str] = None,
    federee: Optional[str] = None,
    tenant_name: Optional[str] = None,
    profiles_file_path: Path = ewc_hub_config.EWC_CLI_PROFILES_PATH,
) -> Dict[str, Optional[str]]:
    """
    Load all profile data (config + credentials) from the single profiles file.

    Parameters
    ----------
    profile : str, optional
        Explicit profile name to load. If None, auto-resolved from federee and tenant_name.
    federee : str, optional
        Federee name, used for auto-resolution if profile is None.
    tenant_name : str, optional
        Tenant name, used for auto-resolution if profile is None.
    profiles_file_path : Path, default to ewc_hub_config.EWC_CLI_PROFILES_PATH
        The path to the file with all profiles.

    Returns
    -------
    dict
        Combined profile data.

    Raises
    ------
    click.Abort
        If the profile cannot be found or cannot be resolved.
    """
    if profile is None:
        if not federee or not tenant_name:
            click.secho(
                "❌ Either 'profile' must be provided or both 'federee' and 'tenant_name'.",
                fg="red",
                bold=True,
            )
            raise click.Abort()
        profile = _resolve_profile(profile, federee, tenant_name)

    cfg = ConfigParser()
    cfg.read(profiles_file_path)

    # Case 1: file missing or empty
    if not os.path.exists(profiles_file_path) or not cfg.sections():
        click.secho(
            "❌ No profiles found.",
            fg="red",
            bold=True,
        )
        click.secho(
            f"Searched in: {profiles_file_path}",
            fg="cyan",
        )
        click.secho(
            "Please run 'ewc login' first to create a profile.",
            fg="yellow",
        )
        raise click.Abort()

    default_profile = ewc_hub_config.EWC_CLI_DEFAULT_PROFILE_NAME
    # Case 2: requested profile missing
    if profile and profile not in cfg:
        if profile != default_profile:
            click.secho(
                f"❌ Profile '{profile}' not found.",
                fg="red",
                bold=True,
            )
            click.secho(
                f"Searched in: {profiles_file_path}",
                fg="cyan",
            )
            if cfg.sections():
                click.secho(
                    f"ℹ️ The {profile} profile does not exist, but other profiles are available:",
                    fg="yellow",
                )
                for name in cfg.sections():
                    click.secho(f"  • {name}", fg="green")

                click.secho(
                    "You can either:",
                    fg="yellow",
                )
                if default_profile in cfg:
                    click.secho(
                        "  • Use the default without --profile",
                        fg="cyan",
                    )
                click.secho(
                    "  • Use one of the existing profiles with --profile <profile_name>",
                    fg="cyan",
                )
                click.secho(
                    "  • Or run 'ewc login' to create a new profile",
                    fg="cyan",
                )

        # Case 3: default profile missing but others exist
        if profile == default_profile and default_profile not in cfg and cfg.sections():
            click.secho(
                "ℹ️ The default profile does not exist, but other profiles are available:",
                fg="yellow",
            )
            for name in cfg.sections():
                click.secho(f"  • {name}", fg="green")

            click.secho(
                "You can either:",
                fg="yellow",
            )
            click.secho(
                "  • Use one of the existing profiles with --profile <profile_name>",
                fg="cyan",
            )
            click.secho(
                "  • Or run 'ewc login' to create the default profile automatically",
                fg="cyan",
            )

        raise click.Abort()

    section = cfg[profile]

    federee = section.get("federee")

    allowed_federees = [f for f in ewc_hub_config.EWC_CLI_SITE_MAP]
    if federee not in allowed_federees:
        raise ClickException(
            f"`{federee}` federee not supported. Check your profiles in ~/.ewccli/profiles. Please use one from the following: {allowed_federees}"
        )

    # Check if SSH keys path exist
    ssh_public_key_path = section.get("ssh_public_key_path")

    if not ssh_public_key_path:
        raise ClickException(
            f"`ssh_public_key_path` key is missing from your profile {profile}, please rerun ewc login."
        )
        
    ssh_private_key_path = section.get("ssh_private_key_path")

    if not ssh_private_key_path:
        raise ClickException(
            f"`ssh_private_key_path` key is missing from your profile {profile}, please rerun ewc login."
        )


    return {
        "profile": profile,
        "federee": federee,
        "tenant_name": section.get("tenant_name"),
        "ssh_public_key_path": ssh_public_key_path,
        "ssh_private_key_path": ssh_private_key_path,
        "region": section.get("region"),
        "token": section.get("token"),
        "application_credential_id": section.get("application_credential_id"),
        "application_credential_secret": section.get("application_credential_secret"),
    }


def generate_random_id(length: int = 10):
    """Generate random ID."""
    characters = string.ascii_letters + string.digits
    random_part = "".join(secrets.choice(characters) for _ in range(length))
    date_part = datetime.now(timezone.utc).strftime("%Y-%m-%dT%H-%M-%SZ")
    return f"{date_part}-{random_part}"


def run_command_from_host(
    description: str,
    command: List[str],
    timeout: Optional[int] = None,
    cwd: Optional[str] = None,
    env: Optional[dict] = None,
    dry_run: bool = False,
) -> Tuple[int, str]:
    """Run command with subprocess."""
    _LOGGER.debug(
        '"%s" -> exec command "%s" with timeout %s', description, command, timeout
    )

    if dry_run:
        return 0, "Dry run. No actions."

    try:
        result = subprocess.run(
            command,
            capture_output=True,
            text=True,  # The output is decoded to a string
            shell=True,
            check=True,  # raise CalledProcessError if non-zero exit code
            cwd=cwd,
            env=env,
        )
        message = ""
        if result.stdout:
            message = f"📤 STDOUT:\n{result.stdout.strip()}"
        return result.returncode, message

    except subprocess.CalledProcessError as e:
        error_message = ""
        if e.stderr:
            error_message = f"📥 STDERR:\n{e.stderr.strip()}"
        return e.returncode, error_message


def run_command_from_host_live(
    description: str,
    command: str,
    timeout: Optional[str] = None,
    cwd: Optional[str] = None,
    env: Optional[dict] = None,
    dry_run: bool = False,
):
    """Run a shell command, streaming output live to the terminal."""
    _LOGGER.info(
        '"%s" -> exec command "%s" with timeout %s', description, command, timeout
    )

    if dry_run:
        return 0, "Dry run. No actions."

    try:
        process = subprocess.Popen(
            command,
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT,
            text=True,  # for automatic decoding (Python 3.7+)
            bufsize=1,  # line-buffered
            cwd=cwd,
            env=env,
            shell=True,
        )
    except Exception as e:
        return 1, f"Failed to start process: {e}"

    def read_first_line(file: Optional[IO[str]]) -> Optional[str]:
        if file is None:
            return None
        return file.readline()

    try:
        while True:
            line = read_first_line(process.stdout)
            if line == "" and process.poll() is not None:
                break
            if line:
                _LOGGER.info(line, end="")

        return process.wait(), "Finishes successfully"

    except Exception as e:
        process.kill()
        return 1, f"\nError running command: {e}"


def download_items(force: bool = False):
    """Download items for the community hub."""
    # URL of the YAML file
    url = ewc_hub_config.EWC_CLI_HUB_ITEMS_URL

    # Path to ~/.ewccli
    config_dir = ewc_hub_config.EWC_CLI_BASE_PATH
    config_dir.mkdir(parents=True, exist_ok=True)

    # Destination file
    item_file = ewc_hub_config.EWC_CLI_HUB_ITEMS_PATH

    if item_file.exists() and not force:
        _LOGGER.debug(f"✅ Items file already exist at {item_file}. Skipping download.")
        return

    if force:
        _LOGGER.debug(
            f"✅ Items file already exist at {item_file}. Force enabled, redownloading it."
        )

    # Download the file
    try:
        # Add a timeout (e.g., 10 seconds)
        response = requests.get(url, timeout=10)
        response.raise_for_status()
        item_file.write_text(response.text)
        _LOGGER.debug(f"Downloaded to: {item_file}")
    except requests.Timeout:
        _LOGGER.error("⚠️ Request timed out.")
    except requests.RequestException as e:
        _LOGGER.error(f"❌ Failed to download file: {e}")


def load_ssh_private_key(encoded_key: Optional[str] = None):
    """Load SSH private key"""
    if encoded_key:
        try:
            private_key = base64.b64decode(encoded_key).decode("utf-8")
            # Use the private_key variable with ssh-add or other SSH tools
            return private_key
        except Exception as e:
            _LOGGER.error(f"Error decoding private key: {e}")
            sys.exit(1)
    else:
        _LOGGER.error("EWC_CLI_ENCODED_SSH_PRIVATE_KEY environment variable not set.")
        sys.exit(1)


def load_ssh_public_key(encoded_key: Optional[str] = None):
    """Load SSH public key"""
    # OpenSSH public keys have the format: <key-type> <base64-data> <comment>
    if encoded_key:
        try:
            public_key = base64.b64decode(encoded_key).decode("utf-8")
            # Use the public_key variable with ssh-add or other SSH tools
            return public_key
        except Exception as e:
            _LOGGER.error(f"Error decoding public key: {e}")
            sys.exit()
    else:
        _LOGGER.error("EWC_CLI_ENCODED_SSH_PUBLIC_KEY environment variable not set.")
        sys.exit(1)


def verify_private_key(private_key: str):
    """Verify SSH private key using cryptography."""
    error = False
    try:
        key_bytes = private_key.encode("utf-8")
        serialization.load_pem_private_key(
            key_bytes,
            password=None,  # If supporting encrypted keys, provide a password
            backend=default_backend(),
        )
        _LOGGER.info("✅ Private key is valid.")
    except ValueError as e:
        _LOGGER.error(f"❌ Invalid SSH key (ValueError): {e}")
        error = True
    except TypeError as e:
        _LOGGER.error(f"❌ SSH key error (TypeError): {e}")
        error = True
    except Exception as e:
        _LOGGER.error(f"❌ Unexpected error while verifying SSH key: {e}")
        error = True
    if error:
        sys.exit(1)


def ssh_keys_match(ssh_private_key_path: str, ssh_public_key_path: str) -> bool:
    """
    Check whether an SSH private key corresponds to a given public key.

    Supports PEM and OpenSSH private key formats and common SSH algorithms
    such as RSA, ECDSA, and Ed25519.

    Args:
        ssh_private_key_path: Path to the SSH private key file.
        ssh_public_key_path: Path to the SSH public key file (.pub).

    Returns:
        True if the public key matches the private key.

    Raises:
        ValueError: If the private key format or algorithm is unsupported.
    """
    # Ensure files exist
    for p in [ssh_private_key_path, ssh_public_key_path]:
        if not Path(p).expanduser().is_file():
            raise ValueError(f"SSH key file does not exist: {p}")

    with open(ssh_private_key_path, "rb") as f:
        private_data = f.read()

    private_key = None

    try:
        private_key = serialization.load_pem_private_key(private_data, password=None)
    except ValueError:
        try:
            private_key = serialization.load_ssh_private_key(private_data, password=None)
        except ValueError:
            raise ValueError("Unsupported or invalid private key format")

    derived_public = private_key.public_key().public_bytes(
        encoding=serialization.Encoding.OpenSSH,
        format=serialization.PublicFormat.OpenSSH
    )

    # Read provided public key and strip comment
    with open(ssh_public_key_path, "r") as f:
        parts = f.read().strip().split()
        if len(parts) < 2:
            raise ValueError(f"Invalid public key format: {ssh_public_key_path}")
        provided_public = " ".join(parts[:2]).encode()

    return derived_public == provided_public


def save_ssh_key(ssh_key, path_key):
    """Store SSH key to the provided path."""
    # Define the file path to save the key
    key_path = os.path.expanduser(path_key)

    # Ensure the .ssh directory exists
    os.makedirs(os.path.dirname(key_path), exist_ok=True)

    # Write the private key to the file with secure permissions
    with open(key_path, "w") as key_file:
        key_file.write(ssh_key)

    # Set file permissions to 0600 (owner read/write only)
    os.chmod(key_path, 0o600)

    _LOGGER.debug(
        f"Key saved temporarely into the container to {key_path} with 0600 permissions."
    )


def save_ssh_keys(
    ssh_public_encoded: Optional[str] = None,
    ssh_private_encoded: Optional[str] = None,
):
    """Store SSH keys provided as encoded strings."""
    if ssh_public_encoded:
        _LOGGER.info("Using encoded public key provided.")
        public_key = load_ssh_public_key(encoded_key=ssh_public_encoded)
        save_ssh_key(
            ssh_key=public_key, path_key=ewc_hub_config.EWC_CLI_PUBLIC_SSH_KEY_PATH
        )

    if ssh_private_encoded:
        _LOGGER.info("Using encoded private key provided.")
        private_key = load_ssh_private_key(encoded_key=ssh_private_encoded)
        verify_private_key(private_key=private_key)
        save_ssh_key(
            ssh_key=private_key, path_key=ewc_hub_config.EWC_CLI_PRIVATE_SSH_KEY_PATH
        )


def generate_ssh_keypair(
    resolved_profile: str
):
    """Generate RSA SSH Key Pair and save to ~/.ssh"""
    private_key = rsa.generate_private_key(
        public_exponent=65537, key_size=2048, backend=default_backend()
    )

    private_key_pem = private_key.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.TraditionalOpenSSL,
        encryption_algorithm=serialization.NoEncryption(),
    )

    public_key = private_key.public_key()
    public_key_ssh = public_key.public_bytes(
        encoding=serialization.Encoding.OpenSSH,
        format=serialization.PublicFormat.OpenSSH,
    )

    # Ensure parent directories exist
    Path(ewc_hub_config.EWC_CLI_HUB_SSH_REPO_PATH).parent.mkdir(parents=True, exist_ok=True)

    ssh_private_key_path = ewc_hub_config.EWC_CLI_HUB_SSH_REPO_PATH / f"{resolved_profile}_id_rsa"
    # Save private key
    with open(ssh_private_key_path, "wb") as f:
        f.write(private_key_pem)

    # Restrict permissions to owner only
    os.chmod(ssh_private_key_path, 0o600)

    ssh_public_key_path = ewc_hub_config.EWC_CLI_HUB_SSH_REPO_PATH / f"{resolved_profile}_id_rsa.pub"
    # Save public key
    with open(ssh_public_key_path, "wb") as f:
        f.write(public_key_ssh)

    # Public key can be world-readable
    os.chmod(ssh_public_key_path, 0o644)

    _LOGGER.info(
        f"SSH key pair generated at {ssh_private_key_path} and {ssh_public_key_path}"
    )

    return ssh_private_key_path, ssh_public_key_path



#!/usr/bin/env python
#
# Package Name: ewccli
# License: GPL-3.0-or-later
# Copyright (c) 2025 EUMETSAT, ECMWF for European Weather Cloud
# See the LICENSE file for more details


"""Test config methods."""

import click
import pytest

# Import your new unified API
from ewccli.utils import (
    save_cli_profile,
    load_cli_profile,
    _resolve_profile,
)


@pytest.fixture
def profile_file_path(tmp_path):
    """Return a temporary path for profiles file."""
    return tmp_path / "profiles"


@pytest.fixture
def ssh_paths(tmp_path):
    """Create fake ssh key paths."""
    priv = tmp_path / "id_rsa"
    pub = tmp_path / "id_rsa.pub"

    priv.write_text("private")
    pub.write_text("public")

    return str(priv), str(pub)


def test_save_and_load_profile(profile_file_path, ssh_paths):
    federee = "EUMETSAT"
    tenant_name = "TeamA"
    token = "tok1"
    app_id = "ID1"
    app_secret = "SECRET1"
    region = "us-east-1"

    ssh_private, ssh_public = ssh_paths

    save_cli_profile(
        federee=federee,
        tenant_name=tenant_name,
        ssh_private_key_path_to_save=ssh_private,
        ssh_public_key_path_to_save=ssh_public,
        token=token,
        application_credential_id=app_id,
        application_credential_secret=app_secret,
        region=region,
        profiles_file_path=str(profile_file_path),
    )

    profile_name = _resolve_profile(None, federee, tenant_name)

    data = load_cli_profile(
        profile=profile_name,
        profiles_file_path=str(profile_file_path),
    )

    assert data["profile"] == profile_name
    assert data["federee"] == federee
    assert data["tenant_name"] == tenant_name
    assert data["token"] == token
    assert data["application_credential_id"] == app_id
    assert data["application_credential_secret"] == app_secret
    assert data["region"] == region
    assert data["ssh_private_key_path"] == ssh_private
    assert data["ssh_public_key_path"] == ssh_public


def test_save_existing_profile_fails(profile_file_path, ssh_paths):
    federee = "EWC2"
    tenant_name = "TeamB"
    ssh_private, ssh_public = ssh_paths

    save_cli_profile(
        federee,
        tenant_name,
        ssh_private,
        ssh_public,
        profiles_file_path=str(profile_file_path),
    )

    with pytest.raises(click.Abort):
        save_cli_profile(
            federee,
            tenant_name,
            ssh_private,
            ssh_public,
            profiles_file_path=str(profile_file_path),
        )


def test_load_missing_profile_raises(profile_file_path):
    with pytest.raises(click.Abort):
        load_cli_profile(
            profile="nonexistent",
            profiles_file_path=str(profile_file_path),
        )

    with pytest.raises(click.Abort):
        load_cli_profile(
            profiles_file_path=str(profile_file_path),
        )


def test_overwrite_profile_not_allowed(profile_file_path, ssh_paths):
    federee = "EWC5"
    tenant_name = "TeamE"
    ssh_private, ssh_public = ssh_paths

    save_cli_profile(
        federee,
        tenant_name,
        ssh_private,
        ssh_public,
        profiles_file_path=str(profile_file_path),
    )

    with pytest.raises(click.Abort):
        save_cli_profile(
            federee,
            tenant_name,
            ssh_private,
            ssh_public,
            profiles_file_path=str(profile_file_path),
        )



#!/usr/bin/env python
#
# Package Name: ewccli
# License: GPL-3.0-or-later
# Copyright (c) 2025 EUMETSAT, ECMWF for European Weather Cloud
# See the LICENSE file for more details


"""CLI EWC Hub: EWC Hub interaction utils methods."""

from typing import Optional, List
from pathlib import Path
from urllib.parse import urlparse

import rich_click as click
from rich.console import Console

from ewccli.enums import HubItemTechnologyAnnotation
from ewccli.logger import get_logger

_LOGGER = get_logger(__name__)


console = Console()


def verify_item_is_deployable(item_info: dict):
    """Verify item is deployable"""
    annotations = item_info.get("annotations")
    check_deployable = 0

    if annotations:
        technology_annotations = list(annotations.get("technology").split(","))

        for tech_annotation in technology_annotations:
            if tech_annotation in [item.value for item in HubItemTechnologyAnnotation]:
                check_deployable += 1

            if check_deployable:
                break

    if not check_deployable:
        _LOGGER.warning("You selected an item that cannot be deployed. Exiting.")
        return False

    return True


def prepare_missing_inputs_error_message(missing_inputs: list[str]):
    """Prepare missing item inputs message."""
    missing_count = len(missing_inputs)
    lines = [f"Missing {missing_count} required item input(s):"]
    lines += [f"- {input_name}" for input_name in missing_inputs]

    return "\n".join(lines)


def extract_annotations(annotations: Optional[dict] = None):
    """Extract annotations from item info."""
    annotations_category: List[str] = []
    annotations_technology: List[str] = []

    if not annotations:
        return annotations_category, annotations_technology

    annotations_category = [
        c.strip() for c in annotations.get("category", "").split(",")
    ]
    annotations_technology = [
        c.strip() for c in annotations.get("technology", "").split(",")
    ]

    return annotations_category, annotations_technology


def classify_source(source: str) -> str:
    """
    Classify a source string as:
      - 'github'     → GitHub HTTPS repo URL compatible with check_github_repo_accessible()
      - 'directory'  → local directory
      - 'unknown'    → neither
  
    Notes:
    - GitHub URLs must be HTTPS and like:
        https://github.com/user/repo
        https://github.com/user/repo.git
    """
    path = Path(source)

    # 1. GitHub URL (only the HTTPS format supported by your check method)
    if is_github_https_url(source):
        return "github"

    # 2. Local directory
    if path.exists() and path.is_dir() and any(path.iterdir()) and path.is_absolute():
        return "directory"

    # 3. Unknown
    raise click.BadParameter(f"Source provided: {source} is not a valid GitHub repo URL or an absolute path to a local directory with content.")


def is_github_https_url(source: str) -> bool:
    """
    Detect ONLY HTTPS GitHub URLs that your check_github_repo_accessible()
    implementation can handle.
    """
    # Strip .git if present
    if source.endswith(".git"):
        source = source[:-4]

    # Normalize slashes
    source = source.rstrip("/")

    parsed = urlparse(source)

    if parsed.scheme != "https":
        return False
    if parsed.netloc != "github.com":
        return False

    parts = parsed.path.strip("/").split("/")
    return len(parts) == 2  # must be exactly owner/repo



#!/usr/bin/env python
#
# Package Name: ewccli
# License: GPL-3.0-or-later
# Copyright (c) 2025 EUMETSAT, ECMWF for European Weather Cloud
# See the LICENSE file for more details

"""Cluster CRD classes."""

from dataclasses import dataclass, field
from typing import Optional

from ewccli.backends.kubernetes.CRDtemplates.commoncrd import GroupVersionResource


@dataclass
class ClusterSpec:
    """
    Specification for a DNS record in the custom resource.

    Attributes:
        siteName (str): The name of the site where this Kubernetes cluster will be deployed.
        clusterName (str): The name of the Kubernetes cluster.
        kubernetesVersion (Optional[str]): The Kubernetes version to deploy.
        nodeCount (Optional[int]): Number of worker nodes in the cluster.
        nodeSize (Optional[str]): Size/flavor of the worker nodes.
        region (Optional[str]): The region where the cluster will be deployed.
    """

    siteName: str
    clusterName: str
    kubernetesVersion: Optional[str] = None
    nodeCount: Optional[int] = None
    nodeSize: Optional[str] = None
    region: Optional[str] = None


@dataclass
class Metadata:
    """
    Metadata for the custom resource, including name, namespace, and optional labels.

    Attributes:
        name (str): Name of the custom resource.
        namespace (Optional[str]): Kubernetes namespace (optional).
        labels (Optional[dict]): Dictionary of key-value labels.
    """

    name: str
    namespace: Optional[str] = None
    labels: Optional[dict] = field(default_factory=dict)


@dataclass
class ClusterConfig:
    """
    Represents the full Cluster record custom resource definition.

    Attributes:
        apiVersion (str): API version of the custom resource.
        kind (str): Kind of the resource (e.g., "Record").
        metadata (Metadata): Resource metadata.
        spec (RecordSpec): Specification of the cluster.
    """

    apiVersion: str
    kind: str
    metadata: Metadata
    spec: ClusterSpec


ClusterGVR = GroupVersionResource(
    group="k8s.europeanweather.cloud", version="v1alpha1", resource="ewcclusters"
)



#!/usr/bin/env python
#
# Package Name: ewccli
# License: GPL-3.0-or-later
# Copyright (c) 2025 EUMETSAT, ECMWF for European Weather Cloud
# See the LICENSE file for more details

"""DNS CRD classes."""

from dataclasses import dataclass, field
from typing import List, Optional

from ewccli.backends.kubernetes.CRDtemplates.commoncrd import GroupVersionResource


@dataclass
class GeoRedundancy:
    """
    Represents georedundancy configuration for a DNS record.

    Attributes:
        enabled (bool): Whether georedundancy is enabled.
        healthEndpoint (str): The health check endpoint path.
        ssl (bool): Whether to use HTTPS for health checks.
    """

    enabled: bool
    healthEndpoint: str
    ssl: bool


@dataclass
class RecordSpec:
    """
    Specification for a DNS record in the custom resource.

    Attributes:
        siteName (str): Name of the site the record belongs to.
        domainName (str): The DNS domain associated with the record.
        recordName (str): Name of the specific DNS record.
        recordType (Optional[str]): Type of DNS record (e.g., A, CNAME, TXT).
        records (tuple): Comma separated DNS record.
        ttl (Optional[int]): Time to live for the DNS record in seconds.
        georedundancy (Optional[GeoRedundancy]): Optional georedundancy settings.
    """

    siteName: str
    domainName: str
    recordName: str
    records: List[str]
    recordType: Optional[str] = None
    ttl: Optional[int] = None
    georedundancy: Optional[GeoRedundancy] = None


@dataclass
class Metadata:
    """
    Metadata for the custom resource, including name, namespace, and optional labels.

    Attributes:
        name (str): Name of the custom resource.
        namespace (Optional[str]): Kubernetes namespace (optional).
        labels (Optional[dict]): Dictionary of key-value labels.
    """

    name: str
    namespace: Optional[str] = None
    labels: Optional[dict] = field(default_factory=dict)


@dataclass
class RecordConfig:
    """
    Represents the full DNS record custom resource definition.

    Attributes:
        apiVersion (str): API version of the custom resource.
        kind (str): Kind of the resource (e.g., "Record").
        metadata (Metadata): Resource metadata.
        spec (RecordSpec): Specification of the DNS record.
    """

    apiVersion: str
    kind: str
    metadata: Metadata
    spec: RecordSpec


RecordGVR = GroupVersionResource(
    group="dns.europeanweather.cloud", version="v1alpha1", resource="records"
)



#!/usr/bin/env python
#
# Package Name: ewccli
# License: GPL-3.0-or-later
# Copyright (c) 2025 EUMETSAT, ECMWF for European Weather Cloud
# See the LICENSE file for more details

"""Common methods for all commands."""

import re
import sys
import os
import getpass
import socket
import time
from pathlib import Path
from typing import Optional
from datetime import datetime, timezone

import yaml
import rich_click as click
from rich.console import Console
from rich.table import Table
from rich import box
from rich.markdown import Markdown
from rich.align import Align

from ewccli.backends.kubernetes.utils import get_reason_from_conditions
from ewccli.enums import HubItemOherAnnotation, HubItemCLIKeys
from ewccli.configuration import config as ewc_hub_config
from ewccli.utils import download_items
from ewccli.logger import get_logger

_LOGGER = get_logger(__name__)


console = Console()


# Global state container
class CommonBackendContext:
    """CommonBackendContext."""

    def __init__(self):
        self.cli_profile = None

class CommonContext:
    """CommonContext."""

    def __init__(self):
        self.cli_profile = None
        self.items = load_hub_items()


def validate_config_name(ctx, param, value):
    """Validate config name."""
    if not value:
        return value

    pattern = r"^[a-zA-Z0-9]+-[a-zA-Z0-9]+-[a-zA-Z0-9]+-[a-zA-Z0-9]+$"
    if not re.match(pattern, value):
        raise click.BadParameter(
            "Config name must be exactly 4 alphanumeric parts separated by dashes (e.g. tenant-federee-east-zone)."
        )
    return value


def login_options(func):
    """Login option for the CLI commands."""
    func = click.option(
        "--profile",
        envvar="EWC_CLI_LOGIN_PROFILE",
        required=False,
        help="EWC CLI profile name",
    )(func)
    return func


def default_keypair_name():
    """Retrieve default keypair name from username."""
    # Safest way to get Linux username
    username = getpass.getuser()
    return f"{username}-ewccli-keypair"


# Compute default now for display purposes
KEYPAIT_DEFAULT = default_keypair_name()


def default_username():
    """Retrieve username runnnig the CLI."""
    # Safest way to get Linux username
    username = getpass.getuser()
    return f"{username}"


def load_hub_items(path_to_catalog: str = ewc_hub_config.EWC_CLI_HUB_ITEMS_PATH) -> dict:
    """Load EWC Hub Items from file."""
    download_items()
    with open(path_to_catalog, "r") as file:
        items_file = yaml.safe_load(file)

        if not items_file:
            _LOGGER.error("items.yaml is empty.")
            sys.exit(1)

        items_spec = items_file.get("spec")

        if not items_spec:
            _LOGGER.error("spec key is missing from items.yaml.")
            sys.exit(1)

        items = items_spec.get("items")

        if not items:
            _LOGGER.error("items key is missing from spec key in items.yaml.")
            sys.exit(1)

        return items


def split_config_name(config_name: str) -> tuple[str, str]:
    """
    Splits config_name into federee and tenant_name.

    Assumes the format: <federee>-<tenant-part1>-<tenant-part2>-<tenant-part3>

    :param config_name: The combined config name string.
    :return: A tuple (federee, tenant_name).
    :raises ValueError: if config_name format is invalid.
    """
    parts = config_name.split("-")
    if len(parts) != 4:
        raise ValueError("config_name must have exactly 4 parts separated by '-'")
    federee = parts[0]
    tenant_name = "-".join(parts[1:])
    return federee, tenant_name


def openstack_options(func):
    """Openstack options for the CLI commands."""
    func = click.option(
        "--auth-url",
        "-url",
        required=False,
        envvar="OS_AUTH_URL",
        type=str,
        help="Openstack Auth URL. (or set env var OS_AUTH_URL)",
    )(func)
    func = click.option(
        "--application-credential-id",
        required=False,
        envvar="OS_APPLICATION_CREDENTIAL_ID",
        type=str,
        help="Openstack Application Credentials ID. (or set env var OS_APPLICATION_CREDENTIAL_ID)",
    )(func)
    func = click.option(
        "--application-credential-secret",
        required=False,
        envvar="OS_APPLICATION_CREDENTIAL_SECRET",
        type=str,
        help="Openstack Application Credentials Secret. (or set env var OS_APPLICATION_CREDENTIAL_SECRET)",
    )(func)

    return func


def _split_env_var(ctx, param, value) -> tuple:
    """Split env var or CLI input into a tuple of unique parameters."""
    if value is None:
        return ()

    if isinstance(value, tuple):
        raw_parameters = value
    else:
        raw_parameters = value.split(",")

    seen = set()
    unique_parameters = []
    for parameter in raw_parameters:
        parameter = parameter.strip()
        if parameter and parameter not in seen:
            seen.add(parameter)
            unique_parameters.append(parameter)

    return tuple(unique_parameters)


def openstack_optional_options(func):
    """Openstack optional options for the CLI commands."""
    func = click.option(
        "--networks",
        "-n",
        required=False,
        envvar="EWC_CLI_OPENSTACK_NETWORKS",
        type=str,
        multiple=True,
        callback=_split_env_var,
        help=(
            "List of networks (comma-separated in env var EWC_CLI_OPENSTACK_NETWORKS "
            "or multiple arguments with the flag)."
        ),
    )(func)
    func = click.option(
        "--security-groups",
        "-sg",
        required=False,
        envvar="EWC_CLI_OPENSTACK_SECURITY_GROUPS",
        type=str,
        multiple=True,
        callback=_split_env_var,
        help=(
            "List of security groups (comma-separated in env var EWC_CLI_OPENSTACK_SECURITY_GROUPS "
            "or multiple arguments with the flag)."
        ),
    )(func)
    func = click.option(
        "--keypair-name",
        "-kp",
        is_flag=False,
        required=False,
        default=default_keypair_name,  # callable, so evaluated at runtime
        envvar="EWC_CLI_OPENSTACK_KEYPAIR_NAME",
        show_default=KEYPAIT_DEFAULT,  # <-- override help display
        type=str,
        help=(
            "Select a name for the keypair in Openstack. "
            "(or set env var EWC_CLI_OPENSTACK_KEYPAIR_NAME)"
        ),
    )(func)
    func = click.option(
        "--image-name",
        "-ig",
        is_flag=False,
        required=False,
        envvar="EWC_CLI_OPENSTACK_IMAGE_NAME",
        show_default=True,
        type=str,
        help="Select image name to be used. (or set env var EWC_CLI_OPENSTACK_IMAGE_NAME)",
    )(func)
    func = click.option(
        "--flavour-name",
        "-fr",
        is_flag=False,
        required=False,
        envvar="EWC_CLI_OPENSTACK_FLAVOUR_NAME",
        show_default=True,
        type=str,
        help="Select a name for the keypair in Openstack. (or set env var EWC_CLI_OPENSTACK_FLAVOUR_NAME)",
    )(func)
    func = click.option(
        "--external-ip",
        is_flag=True,
        default=False,
        envvar="EWC_CLI_EXTERNAL_IP",
        type=bool,
        show_default=True,
        help="Add External IP to the machine.",
    )(func)

    return func


def ssh_options_encoded(func):
    """SSH options encoded for the CLI commands."""
    func = click.option(
        "--ssh-private-encoded",
        required=False,
        envvar="EWC_CLI_ENCODED_SSH_PRIVATE_KEY",
        type=str,
        help="Base64 encoded SSH private key.",
    )(func)
    func = click.option(
        "--ssh-public-encoded",
        required=False,
        envvar="EWC_CLI_ENCODED_SSH_PUBLIC_KEY",
        type=str,
        help="Base64 encoded SSH public key.",
    )(func)

    return func


def validate_path(ctx, param, value):
    """Validate path."""
    if not value:
        return

    try:
        # Expand ~ and resolve to absolute path
        path = Path(value).expanduser().resolve(strict=False)

        # Check if parent directory exists or is creatable
        parent = path.parent
        if not parent.exists():
            try:
                # Try to create it temporarily (then delete)
                parent.mkdir(parents=True, exist_ok=True)
            except Exception as e:
                raise click.BadParameter(f"Cannot create directory '{parent}': {e}")

        # Check for invalid characters (especially on Windows)
        invalid_chars = set('<>:"|?*') if os.name == "nt" else set()
        if any(char in invalid_chars for char in str(path)):
            raise click.BadParameter(f"The path '{path}' contains invalid characters.")

        return path  # Return as a Path object

    except Exception as e:
        raise click.BadParameter(f"Invalid path '{value}': {e}")


def ssh_options(func):
    """SSH options for the CLI commands."""
    func = click.option(
        "--ssh-public-key-path",
        required=False,
        envvar="EWC_CLI_SSH_PUBLIC_KEY_PATH",
        type=Optional[str],
        help="Path to SSH public key.",
        callback=validate_path,
    )(func)
    func = click.option(
        "--ssh-private-key-path",
        required=False,
        envvar="EWC_CLI_SSH_PRIVATE_KEY_PATH",
        type=Optional[str],
        help="Path to SSH private key.",
        callback=validate_path,
    )(func)

    return func


def list_items_table(hub_items: dict):
    """List items in table."""
    table = Table(
        show_header=True,
        header_style="bold green",
        title="EWC HUB Items",
        box=box.MINIMAL_DOUBLE_HEAD,
    )
    table.add_column("Item", overflow="fold")
    table.add_column("Title")
    table.add_column("Version")
    table.add_column("Summary")
    # table.add_column("Description")

    for item, item_v in hub_items.items():
        annotations = item_v.get("annotations")
        if not annotations:
            _LOGGER.warning(f"Filtering {item} as it doesn't contain annotations.")
            continue

        others_annotations = annotations.get("others", "").split(",")

        # Filter items not EWCCLI compatible
        if HubItemOherAnnotation.EWCCLI_COMPATIBLE.value not in others_annotations:
            _LOGGER.warning(
                f"Filtering {item} as this is not compatible with the EWCCLI according to the catalog:"
                f"\n`{HubItemOherAnnotation.EWCCLI_COMPATIBLE.value}` annotation is not in `others` annotations list."
            )
            continue

        table.add_row(
            item,
            item_v.get("displayName"),
            item_v.get("version"),
            item_v.get("summary"),
        )

    console.print(
        table,
        # justify="center"
    )


def show_item_table(hub_item: dict, default_admin_variables_map: Optional[dict] = None):
    """Show item metadata table."""
    table = Table(
        show_header=True,
        show_lines=True,
        header_style="bold green",
        title=f"{hub_item.get('name')} EWC Item Details",
        box=box.MINIMAL_DOUBLE_HEAD,
    )
    table.add_column("Metadata", no_wrap=False, min_width=15)
    table.add_column("Data", no_wrap=False, min_width=15)

    table.add_row("name", hub_item.get("name"))
    table.add_row("version", hub_item.get("version"))
    table.add_row("summary", hub_item.get("summary"))

    for maintainer in hub_item.get("maintainers", {}):
        if maintainer.get("url"):
            table.add_row(
                "maintainer",
                f"[bold]{maintainer.get('name')}:[/bold] [link={maintainer.get('url')}]{maintainer.get('url')}",
            )
        else:
            table.add_row(
                f"maintainer: {maintainer.get('name')}",
                f"[link={maintainer.get('email')}]{maintainer.get('email')}",
            )
    table.add_row("home", f"[link={hub_item.get('home')}]{hub_item.get('home')}")
    table.add_row(
        "license", f"[link={hub_item.get('license')}]{hub_item.get('license')}"
    )

    for annotation, a_v in hub_item.get("annotations", {}).items():
        table.add_row(annotation, a_v)

    md_description = Markdown(hub_item.get("description"))
    table.add_row("description", Align(md_description, align="left", width=80))

    deploy_command_example = f"ewc hub deploy {hub_item.get('name')}"

    if not default_admin_variables_map:
        default_admin_variables_map = {}
    default_admin_variables = [dav for dav in default_admin_variables_map]

    details = ""
    item_info_ewccli = hub_item.get(HubItemCLIKeys.ROOT.value, {})

    for mi in item_info_ewccli.get(HubItemCLIKeys.INPUTS.value, []):
        var_name = mi.get("name")
        default_str = f" (default: {mi['default']})" if "default" in mi else ""
        mandatory = (
            "(mandatory)"
            if ("default" not in mi and var_name not in default_admin_variables)
            else "(optional) "
        )
        details += f"{mandatory} {var_name}: ({mi.get('type')}){default_str}: {mi.get('description')}\n"

        if "default" not in mi and var_name not in default_admin_variables:
            deploy_command_example += f" --item-inputs {var_name}='<>'"

    table.add_row("Inputs", details)

    table.add_row("Deploy command example", deploy_command_example)

    deploy_command_defaults = ""
    if item_info_ewccli.get(HubItemCLIKeys.DEFAULT_IMAGE_NAME.value):
        deploy_command_defaults = f"Image Name: {item_info_ewccli.get(HubItemCLIKeys.DEFAULT_IMAGE_NAME.value)}\n"

    if item_info_ewccli.get(HubItemCLIKeys.DEFAULT_SECURITY_GROUPS.value):
        deploy_command_defaults = f"Security Group/s: {','.join([f for f in item_info_ewccli.get(HubItemCLIKeys.DEFAULT_SECURITY_GROUPS.value, [])])}\n"

    if deploy_command_defaults:
        table.add_row("Deploy command defaults", deploy_command_defaults)

    console.print(
        table,
        # justify="center"
    )


def list_dict_table(title: str, kv: dict):
    """List dictionary in table."""
    table = Table(
        show_header=True,
        header_style="bold green",
        title=title,
        box=box.MINIMAL_DOUBLE_HEAD,
    )
    table.add_column("Keys")
    table.add_column("Value")

    for item_k, item_v in kv.items():
        if isinstance(item_v, list):
            item_v = ",".join(item_v)

        table.add_row(item_k, item_v)

    console.print(
        table,
        # justify="center"
    )


def show_objects(title: str, objects: list, plural: str, namespace: str) -> None:
    """Show objects from kubernetes backend."""
    if not objects:
        click.echo(f"No {plural} found.")
        return None

    table = Table(title=title)
    table.add_column("Name", style="cyan", no_wrap=True)
    table.add_column("Namespace", style="yellow")
    table.add_column("Age", style="green")
    table.add_column("Status", style="magenta")

    now = datetime.now(timezone.utc)

    for item in objects:
        metadata = item.get("metadata", {})
        name = metadata.get("name", "N/A")
        namespace = metadata.get("namespace", namespace)
        creation_ts = metadata.get("creationTimestamp")
        status_obj = item.get("status", {})
        conditions = status_obj.get("conditions", [])

        status = get_reason_from_conditions(conditions)

        age = "N/A"
        if creation_ts:
            created = datetime.fromisoformat(creation_ts.replace("Z", "+00:00"))
            age_seconds = (now - created).total_seconds()
            if age_seconds < 60:
                age = f"{int(age_seconds)}s"
            elif age_seconds < 3600:
                age = f"{int(age_seconds // 60)}m"
            elif age_seconds < 86400:
                age = f"{int(age_seconds // 3600)}h"
            else:
                age = f"{int(age_seconds // 86400)}d"

        table.add_row(name, namespace, age, status)

    console.print(table)


def describe_object(obj: dict) -> None:
    """
    Render a Kubernetes object (CR or built-in) in a kubectl-describe-like table.
    """
    if not obj:
        return None

    def _flatten(d, parent=""):
        """Flatten dicts into key: value (dot notation for nested)."""
        items = []
        for k, v in d.items():
            key = f"{parent}.{k}" if parent else k
            if isinstance(v, dict):
                items.extend(_flatten(v, key))
            elif isinstance(v, list):
                if all(isinstance(i, dict) for i in v):
                    for idx, sub in enumerate(v):
                        items.extend(_flatten(sub, f"{key}[{idx}]"))
                else:
                    items.append((key, ", ".join(str(i) for i in v)))
            else:
                items.append((key, v))
        return items

    # Flatten top-level metadata, spec, status
    sections = {
        "Metadata": obj.get("metadata", {}),
        "Spec": obj.get("spec", {}),
        "Status": obj.get("status", {}),
    }

    # Print header
    click.secho(
        f"Name: {obj.get('metadata', {}).get('name', '<unknown>')}",
        fg="cyan",
        bold=True,
    )
    click.secho(
        f"Namespace: {obj.get('metadata', {}).get('namespace', '<default>')}", fg="cyan"
    )
    click.secho(
        f"Kind: {obj.get('kind', '<unknown>')} | API: {obj.get('apiVersion', '')}",
        fg="cyan",
    )
    click.echo()

    # Print sections
    for section, content in sections.items():
        if not content:
            continue
        click.secho(section, fg="green", bold=True)
        for key, value in _flatten(content):
            click.echo(f"  {key:25} {value}")
        click.echo()


def build_dns_record_name(
    server_name: str, tenancy_name: str, hosting_location: str
) -> str:
    """
    Build a DNS hostname using the ewcloud pattern:
    <machine-name>.<tenancy-name>.<hosting-location>.ewcloud.host
    Source: https://confluence.ecmwf.int/display/EWCLOUDKB/EWC+DNS
    """
    if not all([server_name, tenancy_name, hosting_location]):
        raise ValueError(
            "All arguments (server_name, tenancy_name, hosting_location) are required."
        )

    dns_record_name = f"{server_name}.{tenancy_name}.{hosting_location}.ewcloud.host"
    _LOGGER.debug("Built DNS Record Name: %s", dns_record_name)
    return dns_record_name


def wait_for_dns_record(
    dns_record_name: str, expected_ip: str, interval: int = 60, timeout_minutes: int = 5
) -> bool:
    """
    Waits until the given dns_record_name resolves to the expected IP.
    """
    deadline = time.time() + timeout_minutes * 60
    _LOGGER.info("Waiting for %s to resolve to %s...", dns_record_name, expected_ip)
    _LOGGER.info("⏳ This could take several minutes, grab some snack meanwhile...")

    while time.time() < deadline:
        try:
            resolved_ip = socket.gethostbyname(dns_record_name)

            if resolved_ip == expected_ip:
                _LOGGER.info("Success: %s resolved to %s", dns_record_name, resolved_ip)
                return True
            else:
                _LOGGER.debug(
                    "%s currently resolves to %s (expected %s)",
                    dns_record_name,
                    resolved_ip,
                    expected_ip,
                )

        except socket.gaierror:
            _LOGGER.info(
                f"{dns_record_name} not found in DNS yet. Retrying in {interval} seconds..."
            )

        time.sleep(interval)

    _LOGGER.warning(
        "Timeout: %s did not resolve to %s within %d minutes.",
        dns_record_name,
        expected_ip,
        timeout_minutes,
    )
    return False



#!/usr/bin/env python
#
# Package Name: ewccli
# License: GPL-3.0-or-later
# Copyright (c) 2025 EUMETSAT, ECMWF for European Weather Cloud
# See the LICENSE file for more details


"""CLI EWC: EWC DNS."""

import yaml
import rich_click as click

from ewccli.logger import get_logger
from ewccli.configuration import config as ewc_hub_config
from ewccli.backends.kubernetes.CRDtemplates.dnscrd import RecordGVR
from ewccli.commands.commons import CommonBackendContext
from ewccli.backends.kubernetes.backend_k8s import KubernetesBackend
from ewccli.commands.commons import show_objects, describe_object


_LOGGER = get_logger(__name__)

cb_context = click.make_pass_decorator(CommonBackendContext, ensure=True)


@click.group(name="dns")
@cb_context
def ewc_dns_command(ctx):
    """EWC DNS commands group."""
    token = ctx.cli_config["token"]
    federee = ctx.cli_config["federee"]
    ctx.k8s_backend = KubernetesBackend(
        token=token, host=ewc_hub_config.DEFAULT_KUBERNETES_SERVER.get(federee)
    )


@ewc_dns_command.command()
@cb_context
@click.argument("record_name", type=str)
def delete(ctx, record_name: str):
    """
    Delete a DNS record.

    Example:
         ewc dns delete myrecord
    """
    # Add your backend logic here
    namespace = ctx.cli_config["tenant_name"]
    ctx.k8s_backend.delete_custom_resource(
        group=RecordGVR.group,
        version=RecordGVR.version,
        namespace=namespace,
        plural=RecordGVR.resource,  # this must match the CRD's `spec.names.plural`
        name=record_name,
    )


@ewc_dns_command.command()
@cb_context
def get(ctx):
    """Get list of DNS records."""
    namespace = ctx.cli_config["tenant_name"]
    plural = RecordGVR.resource

    objects = ctx.k8s_backend.list_custom_resources(
        group=RecordGVR.group,
        version=RecordGVR.version,
        namespace=namespace,
        plural=plural,
    )

    show_objects(
        title=f"DNS {plural}", objects=objects, plural=plural, namespace=namespace
    )


@ewc_dns_command.command()
@cb_context
@click.argument("record_name", type=str)
def describe(ctx, record_name: str):
    """Describe a DNS record."""
    namespace = ctx.cli_config["tenant_name"]
    plural = RecordGVR.resource

    object = ctx.k8s_backend.describe_custom_resource(
        name=record_name,
        group=RecordGVR.group,
        version=RecordGVR.version,
        namespace=namespace,
        plural=plural,
    )
    describe_object(object)


@ewc_dns_command.command()
@cb_context
@click.option("--domain-name", required=True, help="Domain name")
@click.option("--record-name", required=True, help="Record name")
@click.option(
    "--records",
    multiple=True,
    required=True,
    help="Record values (can be passed multiple times)",
)
@click.option(
    "--ttl", default=300, show_default=True, type=int, help="Time to live in seconds"
)
@click.option(
    "--record-type",
    default="A",
    show_default=True,
    type=click.Choice(["A", "AAAA", "CNAME", "TXT"], case_sensitive=False),
    help="Record type",
)
@click.option("--geo-enabled", is_flag=True, help="Enable georedundancy")
@click.option(
    "--health-endpoint",
    default="/",
    show_default=True,
    help="Health endpoint for geo redundancy",
)
@click.option(
    "--geo-ssl",
    is_flag=True,
    help="Use HTTPS for health checks",
)
# @click.option(
#     "-f", "--file", type=click.Path(exists=True), help="YAML file to load record spec"
# )
# @click.option(
#     "-o",
#     "--output",
#     type=click.Choice(["yaml", "json", "table"], case_sensitive=False),
#     help="Output format",
# )
@click.option("--dry-run", is_flag=True, help="Simulate creation without applying")
def create(  # noqa: CFQ002
    ctx,
    domain_name,
    record_name,
    records,
    record_type: str,
    ttl: int,
    health_endpoint: str,
    geo_enabled: bool = False,
    geo_ssl: bool = False,
    # file,
    # output,
    dry_run: bool = False,
):
    """Create a DNS record.
Example:
ewc dns create \
--domain-name example.com \
--record-name www \
--records 1.2.3.4,1.2.3.5

    """
    normalized_records = []
    for r in records:
        normalized_records.extend([val.strip() for val in r.split(",") if val.strip()])
    click.echo("Creating DNS record with:")
    click.echo(f"- Domain: {domain_name}")
    click.echo(f"- Record: {record_name}")
    click.echo(f"- Values: {normalized_records}")
    click.echo(f"- TTL: {ttl}, Type: {record_type}")

    if geo_enabled:
        click.echo(
            f"- Geo enabled at {health_endpoint} using {'HTTPS' if geo_ssl else 'HTTP'}"
        )

    # if file:
    #     click.echo(f"- Loading spec from file: {file}")

    if dry_run:
        click.echo("⚠️ Dry run mode enabled. Nothing will be applied.")

    namespace = ctx.cli_config["tenant_name"]
    federee = ctx.cli_config["federee"]

    dns_record_config = {
        "apiVersion": f"{RecordGVR.group}/{RecordGVR.version}",
        "kind": "Record",
        "metadata": {"name": record_name, "namespace": namespace},
        "spec": {
            "siteName": federee,
            "domainName": domain_name,
            "recordName": record_name,
            "recordType": record_type,
            "records": normalized_records,
            "ttl": ttl,
            "georedundancy": {
                "enabled": geo_enabled,
                "healthEndpoint": health_endpoint,
                "ssl": geo_ssl,
            },
        },
    }

    if dry_run:
        # click.echo(json.dumps(dns_record_config, indent=2))
        click.echo(yaml.safe_dump(dns_record_config, sort_keys=False))
    else:
        # Add your backend logic here
        ctx.k8s_backend.create_custom_resource(
            group=RecordGVR.group,
            version=RecordGVR.version,
            namespace=namespace,
            plural=RecordGVR.resource,  # this must match the CRD's `spec.names.plural`
            body=dns_record_config,
        )



#!/usr/bin/env python
#
# Package Name: ewccli
# License: GPL-3.0-or-later
# Copyright (c) 2025 EUMETSAT, ECMWF for European Weather Cloud
# See the LICENSE file for more details


"""Helpers methods for Kubernetes backend."""


def get_status_from_conditions(conditions: list) -> str:
    """
    Extract a human-readable status from conditions list.

    :param conditions: List of condition dicts from CR status.
    :return: Status string, e.g. "Ready: True" or "Synced: True"
    """
    if not conditions:
        return "N/A"
    # Try to find Ready condition first
    ready = next((c for c in conditions if c.get("type") == "Ready"), None)
    if ready:
        return f"{ready.get('type')}: {ready.get('status')}"
    # Otherwise fallback to first condition
    first = conditions[0]
    return f"{first.get('type')}: {first.get('status')}"


def get_reason_from_conditions(conditions: list) -> str:
    """
    Extract the reason from the 'Ready' condition if available,
    otherwise fallback to the first condition's reason.

    :param conditions: List of condition dicts from CR status.
    :return: Reason string, or 'N/A' if not found.
    """
    if not conditions:
        return "N/A"

    # 1. Check if 'Ready' condition exists and has a reason
    ready = next((c for c in conditions if c.get("type") == "Ready"), None)
    if ready and ready.get("reason"):
        return ready["reason"]

    # 2. Look for first condition with status 'False'
    failed = next(
        (c for c in conditions if c.get("status") == "False" and c.get("reason")), None
    )
    if failed:
        return failed["reason"]

    # 3. Fallback to first condition's reason
    return conditions[0].get("reason", "N/A")



#!/usr/bin/env python
#
# Package Name: ewccli
# License: GPL-3.0-or-later
# Copyright (c) 2025 EUMETSAT, ECMWF for European Weather Cloud
# See the LICENSE file for more details


"""CLI EWC: EWC Kubernetes."""

import yaml
import rich_click as click

from ewccli.logger import get_logger
from ewccli.configuration import config as ewc_hub_config
from ewccli.backends.kubernetes.CRDtemplates.clustercrd import ClusterGVR
from ewccli.commands.commons import CommonBackendContext
from ewccli.commands.commons import login_options
from ewccli.commands.commons import split_config_name
from ewccli.backends.kubernetes.backend_k8s import KubernetesBackend
from ewccli.commands.commons import show_objects
from ewccli.utils import load_cli_config


_LOGGER = get_logger(__name__)

cb_context = click.make_pass_decorator(CommonBackendContext, ensure=True)


@click.group(name="k8s")
@cb_context
@login_options
def ewc_k8s_command(ctx, config_name):
    """EWC Kubernetes commands group."""
    if config_name:
        federee, tenant_name = split_config_name(config_name=config_name)
        ctx.cli_config = load_cli_config(tenant_name=tenant_name, federee=federee)

    token = ctx.cli_config["token"]
    federee = ctx.cli_config["federee"]
    ctx.k8s_backend = KubernetesBackend(
        token=token, host=ewc_hub_config.DEFAULT_KUBERNETES_SERVER.get(federee)
    )


@ewc_k8s_command.command()
@cb_context
def get(ctx):
    """Get list of Clusters."""
    namespace = ctx.cli_config["tenant_name"]

    plural = ClusterGVR.resource

    objects = ctx.k8s_backend.list_custom_resources(
        group=ClusterGVR.group,
        version=ClusterGVR.version,
        namespace=namespace,
        plural=plural,
    )

    if not objects:
        click.echo(f"No {plural} found.")
        return

    show_objects(
        title=f"{ClusterGVR.resource}",
        objects=objects,
        plural=plural,
        namespace=namespace,
    )


@ewc_k8s_command.command()
@cb_context
@click.argument("cluster_name", type=str)
def delete(ctx, cluster_name: str):
    """
    Delete a Cluster.

    Example:
         ewc k8s delete clusterName
    """
    # Add your backend logic here
    namespace = ctx.cli_config["tenant_name"]
    ctx.k8s_backend.delete_custom_resource(
        group=ClusterGVR.group,
        version=ClusterGVR.version,
        namespace=namespace,
        plural=ClusterGVR.resource,  # this must match the CRD's `spec.names.plural`
        name=cluster_name,
    )


@ewc_k8s_command.command()
@cb_context
def kubeconfig(ctx):
    """
    Get kubeconfig.
    """
    # Add your backend logic here
    # namespace = ctx.cli_config["tenant_name"]


@ewc_k8s_command.command()
@cb_context
@click.option(
    "--cluster-name", required=True, help="The name of the Kubernetes cluster."
)
@click.option("--k8s-version", required=False, help="The Kubernetes version to deploy.")
@click.option(
    "--node-count",
    required=False,
    type=int,
    help="Number of worker nodes in the cluster.",
)
@click.option("--node-size", required=False, help="Size/flavor of the worker nodes.")
@click.option(
    "--federee", required=False, help="The federee where the cluster will be deployed."
)
@click.option("--dry-run", is_flag=True, help="Simulate creation without applying")
def create(  # noqa: CFQ002
    ctx,
    cluster_name: str,
    k8s_version: str,
    node_count: int,
    node_size: str,
    federee: str,
    dry_run: bool,
):
    """Create a K8s cluster.
Example:
ewc k8s create \
--cluster-name mycluster

    """
    click.echo("Creating K8s cluster with:")
    click.echo(f"- Cluster Name: {cluster_name}")

    if dry_run:
        click.echo("⚠️ Dry run mode enabled. Nothing will be applied.")

    namespace = ctx.cli_config["tenant_name"]
    site_name = ctx.cli_config["federee"]

    # Mandatory
    spec = {
        "siteName": site_name,
        "clusterName": cluster_name,
    }

    if k8s_version:
        spec["kubernetesVersion"] = k8s_version

    if node_count:
        spec["nodeCount"] = node_count

    if node_size:
        spec["nodeSize"] = node_size

    if federee:
        spec["federee"] = federee

    crd_config = {
        "apiVersion": f"{ClusterGVR.group}/{ClusterGVR.version}",
        "kind": "ewcCluster",
        "metadata": {"name": cluster_name, "namespace": namespace},
        "spec": spec,
    }

    if dry_run:
        click.echo(yaml.safe_dump(crd_config, sort_keys=False))
    else:
        # Add your backend logic here
        ctx.k8s_backend.create_custom_resource(
            group=ClusterGVR.group,
            version=ClusterGVR.version,
            namespace=namespace,
            plural=ClusterGVR.resource,  # this must match the CRD's `spec.names.plural`
            body=crd_config,
        )



#!/usr/bin/env python
#
# Package Name: ewccli
# License: GPL-3.0-or-later
# Copyright (c) 2025 EUMETSAT, ECMWF for European Weather Cloud
# See the LICENSE file for more details


"""Tests for EWC commands common methods."""

from unittest import mock
import socket

import pytest
from ewccli.commands.commons import build_dns_record_name, wait_for_dns_record


def test_build_dns_record_name_valid():
    """Should correctly build a DNS record name using the ewcloud pattern."""
    result = build_dns_record_name("server1", "tenancyA", "fra")
    assert result == "server1.tenancyA.fra.ewcloud.host"


@pytest.mark.parametrize(
    "server,tenancy,location",
    [
        (None, "t1", "fra"),
        ("srv", "", "fra"),
        ("srv", "t1", None),
    ],
)
def test_build_dns_record_name_invalid_args(server, tenancy, location):
    """Should raise ValueError when any argument is missing or empty."""
    with pytest.raises(ValueError):
        build_dns_record_name(server, tenancy, location)


@mock.patch("socket.gethostbyname")
@mock.patch("time.sleep", return_value=None)  # skip real sleeping
def test_wait_for_dns_record_success(mock_sleep, mock_gethost):
    """Should return True when DNS resolves to the expected IP."""
    mock_gethost.return_value = "1.2.3.4"
    assert wait_for_dns_record(
        "example.com", "1.2.3.4", interval=1, timeout_minutes=0.01
    )


@mock.patch("socket.gethostbyname", side_effect=socket.gaierror)
@mock.patch("time.sleep", return_value=None)
def test_wait_for_dns_record_timeout(mock_sleep, mock_gethost):
    """Should return False when DNS never resolves within timeout."""
    result = wait_for_dns_record(
        "example.com", "1.2.3.4", interval=1, timeout_minutes=0.01
    )
    assert result is False


@mock.patch("socket.gethostbyname", side_effect=["2.2.2.2", "1.2.3.4"])
@mock.patch("time.sleep", return_value=None)
def test_wait_for_dns_record_eventual_success(mock_sleep, mock_gethost):
    """Should retry until the expected IP is resolved."""
    result = wait_for_dns_record(
        "example.com", "1.2.3.4", interval=1, timeout_minutes=0.05
    )
    assert result is True



#!/usr/bin/env python
#
# Package Name: ewccli
# License: GPL-3.0-or-later
# Copyright (c) 2026 EUMETSAT, ECMWF for European Weather Cloud
# See the LICENSE file for more details


import pytest
from pathlib import Path
from types import SimpleNamespace

from ewccli.backends.openstack.backend_ostack import OpenstackBackend


@pytest.fixture
def backend():
    """Create OpenstackBackend instance without running __init__."""
    return OpenstackBackend.__new__(OpenstackBackend)


def test_ssh_key_matches_openstack_true(tmp_path, backend):
    """
    Test that matching keys return True.
    """

    key = "ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQDtestkey user@test"

    pub_file = tmp_path / "id_rsa.pub"
    pub_file.write_text(key)

    keypair = SimpleNamespace(public_key=key)

    result = backend.ssh_key_matches_openstack(
        public_key_path=str(pub_file),
        keypair=keypair,
    )

    assert result is True


def test_ssh_key_matches_openstack_false(tmp_path, backend):
    """
    Test that different keys return False.
    """

    local_key = "ssh-rsa AAAAB3NzaLOCALKEY user@test"
    cloud_key = "ssh-rsa AAAAB3NzaCLOUDKEY user@test"

    pub_file = tmp_path / "id_rsa.pub"
    pub_file.write_text(local_key)

    keypair = SimpleNamespace(public_key=cloud_key)

    result = backend.ssh_key_matches_openstack(
        public_key_path=str(pub_file),
        keypair=keypair,
    )

    assert result is False


def test_ssh_key_matches_openstack_missing_file(tmp_path, backend):
    """
    Test that missing public key file raises ValueError.
    """

    missing_path = tmp_path / "missing.pub"

    keypair = SimpleNamespace(public_key="ssh-rsa AAAATESTKEY")

    with pytest.raises(ValueError):
        backend.ssh_key_matches_openstack(
            public_key_path=str(missing_path),
            keypair=keypair,
        )


def test_ssh_key_comment_ignored(tmp_path, backend):
    """
    Test that comments at the end of the key are ignored.
    """

    local_key = "ssh-rsa AAAATESTKEY local@machine"
    cloud_key = "ssh-rsa AAAATESTKEY cloud@server"

    pub_file = tmp_path / "id_rsa.pub"
    pub_file.write_text(local_key)

    keypair = SimpleNamespace(public_key=cloud_key)

    result = backend.ssh_key_matches_openstack(
        public_key_path=str(pub_file),
        keypair=keypair,
    )

    assert result is True